from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class UkrTweetToxicityBinaryClassification(AbsTaskClassification):
    input_column_name = "text"
    label_column_name = "toxic"
    train_split = "train"
    metadata = TaskMetadata(
        name="UkrTweetToxicityBinaryClassification",
        description="Filtered Ukrainian Tweets dataset with Toloka.ai platform crowdsourcing task. It contains of 2.5k toxic and 2.5k non-toxic texts.",
        reference="https://huggingface.co/datasets/ukr-detect/ukr-toxicity-dataset",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ukr-Cyrl"],
        main_score="accuracy",
        dataset={
            "path": "mteb/UkrTweetToxicityBinaryClassification_ukr-detect_ukr-toxicity-dataset",
            "revision": "d2638cc529d5d50cecc91b7ae21735fab7b34743",
        },
        date=("2019-01-01", "2025-12-03"),
        domains=["Blog", "Social", "Web"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{dementieva-etal-2024-toxicity,
  address = {Mexico City, Mexico},
  author = {Dementieva, Daryna  and
Khylenko, Valeriia  and
Babakov, Nikolay  and
Groh, Georg},
  booktitle = {Proceedings of the 8th Workshop on Online Abuse and Harms (WOAH 2024)},
  doi = {10.18653/v1/2024.woah-1.19},
  month = jun,
  pages = {244--255},
  publisher = {Association for Computational Linguistics},
  title = {Toxicity Classification in {U}krainian},
  url = {https://aclanthology.org/2024.woah-1.19},
  year = {2024},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SCIDOCSPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SCIDOCS-PL",
        description="SciDocs, a new evaluation benchmark consisting of seven document-level tasks ranging from citation prediction, to document classification and recommendation.",
        reference="https://allenai.org/data/scidocs",
        dataset={
            "path": "mteb/SCIDOCS-PL",
            "revision": "de1caaae2a2880b02023b7fdd36fad58e2470cc1",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2023-05-01", "2024-12-31"),
        domains=["Academic", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["SCIDOCS"],
    )



from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


def _load_data(path: str, splits: str, revision: str | None = None):
    corpus = {}
    queries = {}
    relevant_docs = {}

    dataset = load_dataset(
        path,
        revision=revision,
    )

    for split in splits:
        split_dataset = dataset[split]

        corpus[split] = split_dataset.map(
            lambda x, idx: {
                "id": f"corpus-{split}-{idx}",
                "modality": "image",
            },
            with_indices=True,
            remove_columns=[
                "file_name_index",
                "class",
                "super_class",
                "sub_class",
                "split",
            ],
        )

        queries[split] = split_dataset.map(
            lambda x, idx: {
                "id": f"query-{split}-{idx}",
                "modality": "text",
            },
            with_indices=True,
            remove_columns=[
                "file_name_index",
                "class",
                "super_class",
                "sub_class",
                "split",
            ],
        )
        relevant_docs[split] = {}
        for index in range(len(split_dataset)):
            query_id = f"query-{split}-{index}"
            doc_id = f"corpus-{split}-{index}"
            if query_id not in relevant_docs[split]:
                relevant_docs[split][query_id] = {}
            relevant_docs[split][query_id][doc_id] = 1
    return corpus, queries, relevant_docs


class SciMMIRT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SciMMIRT2IRetrieval",
        description="Retrieve figures and tables based on captions.",
        reference="https://aclanthology.org/2024.findings-acl.746/",
        dataset={
            "path": "mteb/SciMMIR",
            "revision": "2a10b6b1f93cac7084aa809512749272cf0a893a",
        },
        type="Any2AnyRetrieval",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2023-01-01", "2023-12-31"),
        domains=["Academic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{wu2024scimmirbenchmarkingscientificmultimodal,
  archiveprefix = {arXiv},
  author = {Siwei Wu and Yizhi Li and Kang Zhu and Ge Zhang and Yiming Liang and Kaijing Ma and Chenghao Xiao and Haoran Zhang and Bohao Yang and Wenhu Chen and Wenhao Huang and Noura Al Moubayed and Jie Fu and Chenghua Lin},
  eprint = {2401.13478},
  primaryclass = {cs.IR},
  title = {SciMMIR: Benchmarking Scientific Multi-modal Information Retrieval},
  url = {https://arxiv.org/abs/2401.13478},
  year = {2024},
}
""",
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata.dataset["path"],
            splits=self.metadata.eval_splits,
            revision=self.metadata.dataset["revision"],
        )

        self.data_loaded = True



"""Moka AI's Chinese embedding models"""

from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

M3E_CITATION = """@software{MokaMassiveMixedEmbedding,
  author = {Wang Yuxin and Sun Qingxuan and He Sicheng},
  title = {M3E: Moka Massive Mixed Embedding Model},
  year = {2023}
}"""

sent_trf_training_dataset = {
    # derived from datasheets
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "MSMARCO-PL",  # translation not trained on
    "mMARCO-NL",  # translation not trained on
    "NQ",
    "NQ-NL",  # translation not trained on
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",  # translation not trained on
    # not in MTEB
    # "s2orc",
    # "flax-sentence-embeddings/stackexchange_xml",
    # "ms_marco",
    # "gooaq",
    # "yahoo_answers_topics",
    # "code_search_net",
    # "search_qa",
    # "eli5",
    # "snli",
    # "multi_nli",
    # "wikihow",
    # "natural_questions",
    # "trivia_qa",
    # "embedding-data/sentence-compression",
    # "embedding-data/flickr30k-captions",
    # "embedding-data/altlex",
    # "embedding-data/simple-wiki",
    # "embedding-data/QQP",
    # "embedding-data/SPECTER",
    # "embedding-data/PAQ_pairs",
    # "embedding-data/WikiAnswers",
}
medi_dataset = (
    set(
        # not in MTEB:
        # - Super-NI
        # - KILT (https://arxiv.org/abs/2009.02252)
        # - MedMCQA (https://proceedings.mlr.press/v174/pal22a/pal22a.pdf)
    )
    | sent_trf_training_dataset
)
m3e_dataset = {
    "AmazonReviewsClassification",  # Possibly also test, hard to know
    "Ocnli",
    "BQ",
    "LCQMC",
    "MIRACLReranking",
    "PAWSX",
    "DuRetrieval",
    # not in MTEB:
    # - cmrc2018
    # - belle_2m
    # - firefily
    # - alpaca_gpt4
    # - zhihu_kol
    # - hc3_chinese
    # - amazon_reviews_multi (intersects with AmazonReviewsClassification)
    # - qa: Encyclopedia QA dataset
    # - xlsum
    # - wiki_atomic_edit
    # - chatmed_consult
    # - webqa
    # - dureader_robust - DuRetrieval
    # - csl
    # - lawzhidao
    # - CINLID
    # - DuSQL
    # - Zhuiyi-NL2SQL
    # - Cspider
    # - news2016zh
    # - baike2018qa
    # - webtext2019zh
    # - SimCLUE
    # - SQuAD
} | medi_dataset

m3e_base = ModelMeta(
    loader=sentence_transformers_loader,
    name="moka-ai/m3e-base",
    model_type=["dense"],
    languages=["zho-Hans", "eng-Latn"],
    open_weights=True,
    revision="764b537a0e50e5c7d64db883f2d2e051cbe3c64c",
    release_date="2023-06-06",  # first commit
    n_parameters=102267648,
    n_embedding_parameters=16_226_304,
    memory_usage_mb=390,
    embed_dim=768,
    # They don't give a specific license but commercial use is not allowed
    license="https://huggingface.co/moka-ai/m3e-base#%F0%9F%93%9C-license",
    max_tokens=512,
    reference="https://huggingface.co/moka-ai/m3e-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,  # Not published
    training_datasets=m3e_dataset,
    citation=M3E_CITATION,
)

m3e_small = ModelMeta(
    loader=sentence_transformers_loader,
    name="moka-ai/m3e-small",
    model_type=["dense"],
    languages=["zho-Hans", "eng-Latn"],
    open_weights=True,
    revision="44c696631b2a8c200220aaaad5f987f096e986df",
    release_date="2023-06-02",  # first commit
    n_parameters=23953920,
    n_embedding_parameters=10_817_536,
    memory_usage_mb=None,  # Can't be seen on HF page
    embed_dim=512,
    # They don't give a specific license but commercial use is not allowed
    license="https://huggingface.co/moka-ai/m3e-base#%F0%9F%93%9C-license",
    max_tokens=512,
    reference="https://huggingface.co/moka-ai/m3e-small",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,  # Not published
    training_datasets=m3e_dataset,
    citation=M3E_CITATION,
)

m3e_large = ModelMeta(
    loader=sentence_transformers_loader,
    name="moka-ai/m3e-large",
    model_type=["dense"],
    languages=["zho-Hans", "eng-Latn"],
    open_weights=True,
    revision="12900375086c37ba5d83d1e417b21dc7d1d1f388",
    release_date="2023-06-21",  # first commit
    n_parameters=325522432,
    n_embedding_parameters=21_635_072,
    memory_usage_mb=None,  # Can't be seen on HF page
    embed_dim=768,
    # They don't give a specific license but commercial use is not allowed
    license="https://huggingface.co/moka-ai/m3e-base#%F0%9F%93%9C-license",
    max_tokens=512,
    reference="https://huggingface.co/moka-ai/m3e-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,  # Not published
    training_datasets=m3e_dataset,
    citation=M3E_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FinQARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="FinQARetrieval",
        description="A financial retrieval task based on FinQA dataset containing numerical reasoning questions over financial documents. Each query is a financial question requiring numerical computation (e.g., 'What is the percentage change in operating expenses from 2019 to 2020?'), and the corpus contains financial document text with tables and numerical data. The task is to retrieve the correct financial information that enables answering the numerical question. Queries are numerical reasoning questions while the corpus contains financial text passages with embedded tables, figures, and quantitative financial data from earnings reports.",
        reference="https://huggingface.co/datasets/embedding-benchmark/FinQA",
        dataset={
            "path": "embedding-benchmark/FinQA",
            "revision": "bdd1903ce03153129480bfc14b710e3d612c1efd",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2021-12-31"),
        domains=["Financial"],
        task_subtypes=["Question answering"],
        license="mit",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{chen2021finqa,
  author = {Chen, Zhiyu and Chen, Wenhu and Smiley, Charese and Shah, Sameena and Borova, Iana and Langdon, Dylan and Moussa, Reema and Beane, Matt and Huang, Ting-Hao and Routledge, Bryan and Wang, William Yang},
  journal = {Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing},
  title = {FinQA: A Dataset of Numerical Reasoning over Financial Data},
  year = {2021},
}
""",
        prompt={
            "query": "Given a financial numerical reasoning question, retrieve relevant financial data that helps answer the question"
        },
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return

        from datasets import load_dataset

        # Load the three configurations
        corpus_ds = load_dataset(
            self.metadata.dataset["path"],
            "corpus",
            revision=self.metadata.dataset["revision"],
        )["corpus"]
        queries_ds = load_dataset(
            self.metadata.dataset["path"],
            "queries",
            revision=self.metadata.dataset["revision"],
        )["queries"]
        qrels_ds = load_dataset(
            self.metadata.dataset["path"],
            "default",
            revision=self.metadata.dataset["revision"],
        )["test"]

        # Initialize data structures with 'test' split
        corpus = {}
        queries = {}
        relevant_docs = {}

        # Process corpus
        for item in corpus_ds:
            corpus[item["id"]] = {"title": "", "text": item["text"]}

        # Process queries
        for item in queries_ds:
            queries[item["id"]] = item["text"]

        # Process qrels (relevant documents)
        for item in qrels_ds:
            query_id = item["query-id"]
            if query_id not in relevant_docs:
                relevant_docs[query_id] = {}
            relevant_docs[query_id][item["corpus-id"]] = int(item["score"])

        # Organize data by splits as expected by MTEB
        self.corpus = {"test": corpus}
        self.queries = {"test": queries}
        self.relevant_docs = {"test": relevant_docs}

        self.data_loaded = True



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import ASTFeatureExtractor, ASTModel

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput

logger = logging.getLogger(__name__)


class ASTWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device

        self.feature_extractor = ASTFeatureExtractor.from_pretrained(model_name)
        self.model = ASTModel.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.model.eval()
        self.sampling_rate = self.feature_extractor.sampling_rate

    @torch.no_grad()
    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)
        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = []
            for a in batch["audio"]:
                array = a["array"]
                # Ensure minimum length for AST feature extractor (window size is 400)
                min_samples = 401  # Just above the window size
                if len(array) < min_samples:
                    padding = torch.zeros(min_samples - len(array))
                    array = torch.cat([array, padding])

                audio_arrays.append(array.numpy())

            features = self.feature_extractor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                truncation=True,
                padding=True,
            ).to(self.device)

            outputs = self.model(**features)

            # AST's pooled output is the [CLS] token embedding
            embeddings = outputs.pooler_output
            all_embeddings.append(embeddings.cpu().detach())
        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("ASTWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


# Model metadata
ast_audioset = ModelMeta(
    loader=ASTWrapper,
    name="MIT/ast-finetuned-audioset-10-10-0.4593",
    languages=["eng-Latn"],
    open_weights=True,
    revision="f826b80d28226b62986cc218e5cec390b1096902",
    release_date="2021-07-08",
    max_tokens=None,
    n_parameters=86_600_000,
    memory_usage_mb=330,
    embed_dim=768,
    license="apache-2.0",
    reference="https://huggingface.co/MIT/ast-finetuned-audioset-10-10-0.4593",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/YuanGongND/ast",
    public_training_data="https://research.google.com/audioset/dataset/index.html",
    training_datasets=set(),  # "AudioSet": ["train"]},
    modalities=["audio"],
    citation="""
@misc{gong2021astaudiospectrogramtransformer,
      title={AST: Audio Spectrogram Transformer},
      author={Yuan Gong and Yu-An Chung and James Glass},
      year={2021},
      eprint={2104.01778},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2104.01778},
}""",
)



from __future__ import annotations

import logging
import warnings
from typing import TYPE_CHECKING

import numpy as np
import torch

from mteb._requires_package import requires_package
from mteb.models.model_meta import ScoringFunction

if TYPE_CHECKING:
    from collections.abc import Callable

    import faiss

    from mteb.models.models_protocols import EncoderProtocol
    from mteb.types import Array, TopRankedDocumentsType


logger = logging.getLogger(__name__)


class FaissSearchIndex:
    """FAISS-based backend for encoder-based search.

    Supports both full-corpus retrieval and reranking (via `top_ranked`).

    Notes:
        - Stores *all* embeddings in memory (IndexFlatIP or IndexFlatL2).
        - Expects embeddings to be normalized if cosine similarity is desired.
    """

    _normalize: bool = False

    def __init__(self, model: EncoderProtocol) -> None:
        requires_package(
            self,
            "faiss",
            "FAISS-based search",
            install_instruction="pip install mteb[faiss-cpu]",
        )

        from faiss import IndexFlatIP, IndexFlatL2

        # https://github.com/facebookresearch/faiss/wiki/Faiss-indexes
        if model.mteb_model_meta.similarity_fn_name is ScoringFunction.DOT_PRODUCT:
            self.index_type = IndexFlatIP
        elif model.mteb_model_meta.similarity_fn_name is ScoringFunction.COSINE:
            self.index_type = IndexFlatIP
            self._normalize = True
        elif model.mteb_model_meta.similarity_fn_name is ScoringFunction.EUCLIDEAN:
            self.index_type = IndexFlatL2
        else:
            raise ValueError(
                f"FAISS backend does not support similarity function {model.mteb_model_meta.similarity_fn_name}. "
                f"Available: {ScoringFunction.DOT_PRODUCT}, {ScoringFunction.COSINE}."
            )

        self.idxs: list[str] = []
        self.index: faiss.Index | None = None

    def add_documents(self, embeddings: Array, idxs: list[str]) -> None:
        """Add all document embeddings and their IDs to FAISS index."""
        import faiss

        if isinstance(embeddings, torch.Tensor):
            embeddings = embeddings.detach().cpu().numpy()

        embeddings = embeddings.astype(np.float32)
        self.idxs.extend(idxs)

        if self._normalize:
            faiss.normalize_L2(embeddings)

        dim = embeddings.shape[1]
        if self.index is None:
            self.index = self.index_type(dim)

        self.index.add(embeddings)
        logger.info(f"FAISS index built with {len(idxs)} vectors of dim {dim}.")

    def search(
        self,
        embeddings: Array,
        top_k: int,
        similarity_fn: Callable[[Array, Array], Array],
        top_ranked: TopRankedDocumentsType | None = None,
        query_idx_to_id: dict[int, str] | None = None,
    ) -> tuple[list[list[float]], list[list[int]]]:
        """Search using FAISS."""
        import faiss

        if self.index is None:
            raise ValueError("No index built. Call add_document() first.")

        if isinstance(embeddings, torch.Tensor):
            embeddings = embeddings.detach().cpu().numpy()

        if self._normalize:
            faiss.normalize_L2(embeddings)

        if top_ranked is not None:
            if query_idx_to_id is None:
                raise ValueError("query_idx_to_id must be provided when reranking.")

            similarities, ids = self._reranking(
                embeddings,
                top_k,
                top_ranked=top_ranked,
                query_idx_to_id=query_idx_to_id,
            )
        else:
            similarities, ids = self.index.search(embeddings.astype(np.float32), top_k)
            similarities = similarities.tolist()
            ids = ids.tolist()

        if issubclass(self.index_type, faiss.IndexFlatL2):
            similarities = (-np.sqrt(np.maximum(similarities, 0))).tolist()

        return similarities, ids

    def _reranking(
        self,
        embeddings: Array,
        top_k: int,
        top_ranked: TopRankedDocumentsType,
        query_idx_to_id: dict[int, str],
    ) -> tuple[list[list[float]], list[list[int]]]:
        doc_id_to_idx = {doc_id: i for i, doc_id in enumerate(self.idxs)}
        scores_all: list[list[float]] = []
        idxs_all: list[list[int]] = []

        for query_idx, query_emb in enumerate(embeddings):
            query_id = query_idx_to_id[query_idx]
            ranked_ids = top_ranked.get(query_id)
            if not ranked_ids:
                msg = f"No top-ranked documents for query {query_id}"
                logger.warning(msg)
                warnings.warn(msg)
                scores_all.append([])
                idxs_all.append([])
                continue

            candidate_indices = [doc_id_to_idx[doc_id] for doc_id in ranked_ids]
            d = self.index.d  # type: ignore[union-attr]
            candidate_embs = np.vstack(
                [self.index.reconstruct(idx) for idx in candidate_indices]  # type: ignore[union-attr]
            )
            sub_reranking_index = self.index_type(d)
            sub_reranking_index.add(candidate_embs)

            # Search returns scores and indices in one call
            scores, local_indices = sub_reranking_index.search(
                query_emb.reshape(1, -1).astype(np.float32),
                min(top_k, len(candidate_indices)),
            )
            # faiss will output 2d arrays even for single query
            scores_all.append(scores[0].tolist())
            idxs_all.append(local_indices[0].tolist())

        return scores_all, idxs_all

    def clear(self) -> None:
        """Clear all stored documents and embeddings from the backend."""
        self.index = None
        self.idxs = []



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class VoxLingua107Top10(AbsTaskClassification):
    metadata = TaskMetadata(
        name="VoxLingua107_Top10",
        description="Spoken Language Identification for a given audio samples (10 classes/languages)",
        reference="https://huggingface.co/datasets/silky1708/VoxLingua107-Top-10",
        dataset={
            "path": "mteb/voxlingua107-top10",
            "revision": "d934546d059e16c9a4669adbd518e0fa86a69ff0",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2020-01-01", "2020-12-31"),
        domains=["Speech"],
        task_subtypes=["Spoken Language Identification"],
        license="cc-by-4.0",
        annotations_creators="automatic-and-reviewed",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",  # from youtube
        bibtex_citation=r"""
@misc{valk2020voxlingua107datasetspokenlanguage,
  archiveprefix = {arXiv},
  author = {Jörgen Valk and Tanel Alumäe},
  eprint = {2011.12998},
  primaryclass = {eess.AS},
  title = {VoxLingua107: a Dataset for Spoken Language Recognition},
  url = {https://arxiv.org/abs/2011.12998},
  year = {2020},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    is_cross_validation: bool = True



from mteb.abstasks import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class VoxPopuliAccentPairClassification(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="VoxPopuliAccentPairClassification",
        description="Classifying same or different regional accent of English (Empty Audio Samples filtered out)",
        reference="https://aclanthology.org/2021.acl-long.80/",
        dataset={
            "path": "mteb/VoxPopuliAccentPairClassificationFiltered",
            "revision": "ac4587795abbac156982e41775e1428f5933a32e",
        },
        type="AudioPairClassification",
        category="a2a",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="max_ap",
        date=("2021-01-01", "2021-08-01"),
        domains=["Spoken"],
        task_subtypes=["Emotion classification"],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{wang-etal-2021-voxpopuli,
  address = {Online},
  author = {Wang, Changhan  and
Riviere, Morgane  and
Lee, Ann  and
Wu, Anne  and
Talnikar, Chaitanya  and
Haziza, Daniel  and
Williamson, Mary  and
Pino, Juan  and
Dupoux, Emmanuel},
  booktitle = {Proceedings of the 59th Annual Meeting of the Association for Computational Linguistics and the 11th International Joint Conference on Natural Language Processing (Volume 1: Long Papers)},
  doi = {10.18653/v1/2021.acl-long.80},
  editor = {Zong, Chengqing  and
Xia, Fei  and
Li, Wenjie  and
Navigli, Roberto},
  month = aug,
  pages = {993--1003},
  publisher = {Association for Computational Linguistics},
  title = {{V}ox{P}opuli: A Large-Scale Multilingual Speech Corpus for Representation Learning, Semi-Supervised Learning and Interpretation},
  url = {https://aclanthology.org/2021.acl-long.80/},
  year = {2021},
}
""",
    )

    input1_column_name: str = "audio1"
    input2_column_name: str = "audio2"
    label_column_name: str = "label"



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "en-as": ["eng-Latn", "asm-Beng"],
    "en-bn": ["eng-Latn", "ben-Beng"],
    "en-gu": ["eng-Latn", "guj-Gujr"],
    "en-hi": ["eng-Latn", "hin-Deva"],
    "en-kn": ["eng-Latn", "kan-Knda"],
    "en-ml": ["eng-Latn", "mal-Mlym"],
    "en-mr": ["eng-Latn", "mar-Deva"],
    "en-or": ["eng-Latn", "ory-Orya"],
    "en-pa": ["eng-Latn", "pan-Guru"],
    "en-ta": ["eng-Latn", "tam-Taml"],
    "en-te": ["eng-Latn", "tel-Telu"],
    "en-ur": ["eng-Latn", "urd-Arab"],
}


class IndicCrosslingualSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="IndicCrosslingualSTS",
        dataset={
            "path": "mteb/IndicCrosslingualSTS",
            "revision": "f0366eb5a20087355c0e131162bbed943ba54b51",
        },
        description="This is a Semantic Textual Similarity testset between English and 12 high-resource Indic languages.",
        reference="https://huggingface.co/datasets/jaygala24/indic_sts",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="cosine_spearman",
        date=("2021-04-30", "2021-06-09"),
        domains=[
            "News",
            "Non-fiction",
            "Web",
            "Spoken",
            "Government",
            "Written",
            "Spoken",
        ],
        task_subtypes=[],
        license="cc0-1.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@article{10.1162/tacl_a_00452,
  author = {Ramesh, Gowtham and Doddapaneni, Sumanth and Bheemaraj, Aravinth and Jobanputra, Mayank and AK, Raghavan and Sharma, Ajitesh and Sahoo, Sujit and Diddee, Harshita and J, Mahalakshmi and Kakwani, Divyanshu and Kumar, Navneet and Pradeep, Aswin and Nagaraj, Srihari and Deepak, Kumar and Raghavan, Vivek and Kunchukuttan, Anoop and Kumar, Pratyush and Khapra, Mitesh Shantadevi},
  doi = {10.1162/tacl_a_00452},
  eprint = {https://direct.mit.edu/tacl/article-pdf/doi/10.1162/tacl\\_a\\_00452/1987010/tacl\\_a\\_00452.pdf},
  issn = {2307-387X},
  journal = {Transactions of the Association for Computational Linguistics},
  month = {02},
  pages = {145-162},
  title = {{Samanantar: The Largest Publicly Available Parallel Corpora Collection for 11 Indic Languages}},
  url = {https://doi.org/10.1162/tacl\\_a\\_00452},
  volume = {10},
  year = {2022},
}
""",
    )

    min_score = 0
    max_score = 5



from .cedr_classification import CEDRClassification
from .ru_toixic_multilabelclassification_okmlcup import (
    RuToxicOKMLCUPMultilabelClassification,
)
from .sensitive_topics_classification import SensitiveTopicsClassification

__all__ = [
    "CEDRClassification",
    "RuToxicOKMLCUPMultilabelClassification",
    "SensitiveTopicsClassification",
]



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata

_EVAL_SPLIT = "test"


class STSES(AbsTaskSTS):
    metadata = TaskMetadata(
        name="STSES",
        dataset={
            "path": "mteb/STSES",
            "revision": "fe7158e70012087b5a692ce175226a0d213936ad",
        },
        description="Spanish test sets from SemEval-2014 (Agirre et al., 2014) and SemEval-2015 (Agirre et al., 2015)",
        reference="https://huggingface.co/datasets/PlanTL-GOB-ES/sts-es",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["spa-Latn"],
        main_score="cosine_spearman",
        date=("2014-01-01", "2015-12-31"),
        domains=["News", "Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{agirre2014semeval,
  author = {Agirre, Eneko and Banea, Carmen and Cardie, Claire and Cer, Daniel M and Diab, Mona T and Gonzalez-Agirre, Aitor and Guo, Weiwei and Mihalcea, Rada and Rigau, German and Wiebe, Janyce},
  booktitle = {SemEval@ COLING},
  pages = {81--91},
  title = {SemEval-2014 Task 10: Multilingual Semantic Textual Similarity.},
  year = {2014},
}

@inproceedings{agirre2015semeval,
  author = {Agirre, Eneko and Banea, Carmen and Cardie, Claire and Cer, Daniel and Diab, Mona and Gonzalez-Agirre, Aitor and Guo, Weiwei and Lopez-Gazpio, Inigo and Maritxalar, Montse and Mihalcea, Rada and others},
  booktitle = {Proceedings of the 9th international workshop on semantic evaluation (SemEval 2015)},
  pages = {252--263},
  title = {Semeval-2015 task 2: Semantic textual similarity, english, spanish and pilot on interpretability},
  year = {2015},
}
""",
    )

    min_score = 0
    max_score = 5



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import Data2VecAudioModel, Wav2Vec2FeatureExtractor

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


class Data2VecAudioWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        # Data2Vec Audio also uses Wav2Vec2 feature extractor
        self.feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained(model_name)
        self.model = Data2VecAudioModel.from_pretrained(
            model_name, revision=revision
        ).to(self.device)
        self.model.eval()

        self.sampling_rate = self.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]
            feature_inputs = self.feature_extractor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="longest",
                truncation=True,
                max_length=int(self.max_audio_length_seconds * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                outputs = self.model(
                    feature_inputs.input_values,
                    attention_mask=feature_inputs.attention_mask,
                    output_hidden_states=True,
                )

                last_hidden_state = outputs.last_hidden_state

                # Apply attention-masked pooling to exclude padding tokens
                batch_size, hidden_seq_len, hidden_size = last_hidden_state.shape
                device = last_hidden_state.device

                # Calculate proper hidden lengths based on input attention mask
                input_lengths = feature_inputs.attention_mask.sum(dim=1)
                downsample_ratio = feature_inputs.input_values.shape[1] / hidden_seq_len
                hidden_lengths = (input_lengths.float() / downsample_ratio).long()
                hidden_lengths = torch.clamp(hidden_lengths, min=0, max=hidden_seq_len)

                # Create attention mask for hidden states
                seq_range = torch.arange(hidden_seq_len, device=device).unsqueeze(0)
                hidden_attention_mask = (seq_range < hidden_lengths.unsqueeze(1)).to(
                    last_hidden_state.dtype
                )

                # Apply masked mean pooling
                hidden_attention_mask = hidden_attention_mask.unsqueeze(-1)
                masked_embeddings = last_hidden_state * hidden_attention_mask
                valid_tokens = hidden_attention_mask.sum(dim=1)
                embeddings = masked_embeddings.sum(dim=1) / valid_tokens.clamp(min=1e-9)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("Data2VecAudioWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


# Base model
data2vec_audio_base = ModelMeta(
    loader=Data2VecAudioWrapper,
    name="facebook/data2vec-audio-base-960h",
    languages=["eng-Latn"],
    open_weights=True,
    revision="32331f3123e703528918aa688a9a38232d58c872",
    release_date="2022-02-07",  # Paper release date
    max_tokens=None,
    n_parameters=93_164_288,
    memory_usage_mb=355,
    embed_dim=768,
    license="mit",
    reference="https://huggingface.co/facebook/data2vec-audio-base-960h",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/facebookresearch/fairseq/tree/main/examples/data2vec",
    public_training_data="https://www.openslr.org/12",  # Link to LibriSpeech Dataset
    training_datasets=set(),  # "LibriSpeech": ["train"]},
    modalities=["audio"],
    citation="""
@misc{baevski2022data2vecgeneralframeworkselfsupervised,
    title={data2vec: A General Framework for Self-supervised Learning in Speech, Vision and Language},
    author={Alexei Baevski and Wei-Ning Hsu and Qiantong Xu and Arun Babu and Jiatao Gu and Michael Auli},
    year={2022},
    eprint={2202.03555},
    archivePrefix={arXiv},
    primaryClass={cs.LG},
    url={https://arxiv.org/abs/2202.03555},
}""",
)

data2vec_audio_large = ModelMeta(
    loader=Data2VecAudioWrapper,
    name="facebook/data2vec-audio-large-960h",
    languages=["eng-Latn"],
    open_weights=True,
    revision="27aba26eed532b86dcd0f17284a0307de4b51f39",
    release_date="2022-02-07",  # Paper release date
    max_tokens=None,
    n_parameters=313_276_416,
    memory_usage_mb=1195,
    embed_dim=1024,
    license="mit",
    reference="https://huggingface.co/facebook/data2vec-audio-large-960h",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/facebookresearch/fairseq/tree/main/examples/data2vec",
    public_training_data="https://www.openslr.org/12",  # Link to LibriSpeech Dataset
    training_datasets=set(),  # "LibriSpeech": ["train"]},
    modalities=["audio"],
    citation="""
@misc{baevski2022data2vecgeneralframeworkselfsupervised,
    title={data2vec: A General Framework for Self-supervised Learning in Speech, Vision and Language},
    author={Alexei Baevski and Wei-Ning Hsu and Qiantong Xu and Arun Babu and Jiatao Gu and Michael Auli},
    year={2022},
    eprint={2202.03555},
    archivePrefix={arXiv},
    primaryClass={cs.LG},
    url={https://arxiv.org/abs/2202.03555},
}""",
)



"""Test cases for the ResultCache class in the mteb.cache module."""

import gzip
from pathlib import Path
from typing import cast
from unittest.mock import Mock, patch

import numpy as np
import pytest
import requests

import mteb
from mteb.cache import LoadExperimentEnum, ResultCache
from mteb.results import TaskResult
from tests.mock_tasks import MockMultilingualClusteringTask, MockRetrievalTask

test_cache_path = Path(__file__).parent / "mock_mteb_cache"


def test_result_cache() -> None:
    cache = ResultCache(cache_path=test_cache_path)

    assert cache.has_remote is True, "Cache should not have a remote repository"

    # load known results from the cache
    result = cache.load_task_result(
        "BornholmBitextMining",
        "sentence-transformers/all-MiniLM-L6-v2",
        model_revision="8b3219a92973c328a8e22fadcfa821b5dc75636a",
        raise_if_not_found=True,
    )
    result = cache.load_task_result(
        "BornholmBitextMining",
        "sentence-transformers/all-MiniLM-L6-v2",
        raise_if_not_found=True,
    )

    assert isinstance(result, TaskResult), "Loaded result should be a TaskResult"
    assert result.task_name == "BornholmBitextMining", "Task name should match"


def test_get_cache_path() -> None:
    cache = ResultCache(cache_path=test_cache_path)
    paths = cache.get_cache_paths(require_model_meta=False, include_remote=False)

    assert isinstance(paths, list), "Cache paths should be a list"
    assert isinstance(paths[0], Path), "Cache paths should be a list of Paths"

    paths_w_meta = cache.get_cache_paths(require_model_meta=True, include_remote=False)
    assert len(paths_w_meta) < len(paths), (
        "Paths with model meta should be fewer than without"
    )

    paths_w_remote = cache.get_cache_paths(
        include_remote=True, require_model_meta=False
    )

    assert len(paths_w_remote) > len(paths), (
        "Paths with remote should be at least as many as without"
    )

    known_model = "sentence-transformers/average_word_embeddings_levy_dependency"

    paths_for_model = cache.get_cache_paths(
        models=[known_model], require_model_meta=False
    )
    assert len(paths_for_model) > 0, "Should return paths for the specified model"


def test_get_models_and_tasks() -> None:
    cache = ResultCache(cache_path=test_cache_path)

    models = cache.get_models()
    assert isinstance(models, list), "Models should be a list"
    assert isinstance(models[0], tuple) and len(models[0]) == 2, (
        "Models should be a list of tuples (model_name, model_revision)"
    )

    tasks = cache.get_task_names()
    assert isinstance(tasks, list), "Tasks should be a list"
    assert isinstance(tasks[0], str), "Tasks should be a list of task names"

    known_model = "sentence-transformers__average_word_embeddings_levy_dependency"
    known_revision = "6d9c09a789ad5dd126b476323fccfeeafcd90509"

    assert known_model in [mdl[0] for mdl in models], (
        "Known model should be in the results"
    )
    assert known_revision in [mdl[1] for mdl in models if mdl[0] == known_model], (
        "Known revision should be in the results for the known model"
    )


def test_no_duplicates_in_models() -> None:
    """Test that get_models() returns no duplicates (issue #3173)."""
    cache = ResultCache(cache_path=test_cache_path)

    models = cache.get_models()

    # Check that there are no duplicates
    assert len(models) == len(set(models)), (
        f"get_models() returned {len(models)} models but {len(set(models))} unique models. "
        "There should be no duplicates."
    )


def test_no_duplicates_in_tasks() -> None:
    """Test that get_task_names() returns no duplicates (issue #3173)."""
    cache = ResultCache(cache_path=test_cache_path)

    tasks = cache.get_task_names()

    # Check that there are no duplicates
    assert len(tasks) == len(set(tasks)), (
        f"get_task_names() returned {len(tasks)} tasks but {len(set(tasks))} unique tasks. "
        "There should be no duplicates."
    )


def test_load_results():
    cache = ResultCache(cache_path=test_cache_path)

    results = cache.load_results()

    known_model = "sentence-transformers/average_word_embeddings_levy_dependency"
    known_revision = "6d9c09a789ad5dd126b476323fccfeeafcd90509"

    assert known_model in [res.model_name for res in results]
    assert known_revision in [
        res.model_revision for res in results if res.model_name == known_model
    ], "Known revision should be in the results"


def test_load_result_specific_model():
    cache = ResultCache(cache_path=test_cache_path)

    model = "sentence-transformers/average_word_embeddings_levy_dependency"
    results = cache.load_results(models=[model], require_model_meta=False)

    model_names = {mdl_res.model_name for mdl_res in results.model_results}
    assert len(model_names) == 1, "Should only have one model in the results"
    assert model in model_names, "Model should be in the results"


def test_filter_with_modelmeta():
    cache = ResultCache(cache_path=test_cache_path)

    base = test_cache_path / "results"
    model_meta = mteb.get_model_meta("sentence-transformers/all-MiniLM-L6-v2")

    model_name = model_meta.model_name_as_path()
    model_revision_1 = model_meta.revision
    model_revision_1 = cast("str", model_revision_1)
    sample_paths = [
        base / model_name / model_revision_1 / "task1.json",
        base / model_name / model_revision_1 / "task2.json",
        base / model_name / "revision" / "task1.json",
        base / "not_existing_model" / "revision" / "task2.json",
    ]

    filtered = cache._filter_paths_by_model_and_revision(sample_paths, [model_meta])

    expected = {
        (
            "sentence-transformers__all-MiniLM-L6-v2",
            "8b3219a92973c328a8e22fadcfa821b5dc75636a",
        )
    }
    actual = {(p.parent.parent.name, p.parent.name) for p in filtered}
    assert actual == expected


def test_filter_with_string_models():
    cache = ResultCache(cache_path=test_cache_path)

    base = test_cache_path / "results"
    model_meta = mteb.get_model_meta("sentence-transformers/all-MiniLM-L6-v2")

    model_name = model_meta.model_name_as_path()
    model_revision_1 = model_meta.revision
    model_revision_1 = cast("str", model_revision_1)
    sample_paths = [
        base / model_name / model_revision_1 / "task1.json",
        base / model_name / model_revision_1 / "task2.json",
        base / model_name / "revision" / "task1.json",
        base / "not_existing_model" / "revision" / "task2.json",
    ]

    filtered = cache._filter_paths_by_model_and_revision(sample_paths, [model_name])

    expected = {
        (
            "sentence-transformers__all-MiniLM-L6-v2",
            "8b3219a92973c328a8e22fadcfa821b5dc75636a",
        ),
        ("sentence-transformers__all-MiniLM-L6-v2", "revision"),
    }
    actual = {(p.parent.parent.name, p.parent.name) for p in filtered}
    assert actual == expected


def test_cache_filter_languages():
    cache = ResultCache(cache_path=test_cache_path)

    task = MockMultilingualClusteringTask()
    results = cache.load_results(
        tasks=[task],
        validate_and_filter=True,
    )
    assert len(results.model_results[0].task_results[0].scores["test"]) == 2
    task = task.filter_languages(["eng"])
    eng_results = cache.load_results(tasks=[task], validate_and_filter=True)
    assert len(eng_results.model_results[0].task_results[0].scores["test"]) == 1


def test_cache_load_different_subsets():
    cache = ResultCache(cache_path=test_cache_path)

    task = mteb.get_task(
        "BelebeleRetrieval", hf_subsets=["acm_Arab-acm_Arab", "nld_Latn-nld_Latn"]
    )
    model1 = mteb.get_model_meta(
        "sentence-transformers/all-MiniLM-L6-v2"
    )  # model have only arab subset results
    model2 = mteb.get_model_meta(
        "mteb/baseline-random-encoder"
    )  # model have all subsets results

    result1 = cache.load_results(
        models=[
            model1,
        ],
        tasks=[task],
    )
    result2 = cache.load_results(
        models=[
            model2,
        ],
        tasks=[task],
    )
    assert len(result1.model_results[0].task_results[0].scores["test"]) == 1
    assert len(result2.model_results[0].task_results[0].scores["test"]) == 2

    assert result1.model_results[0].task_results[0].get_score() == 0.01568
    assert result2.model_results[0].task_results[0].get_score() == 0.01035

    result1 = cache.load_results(
        models=[
            model1,
        ],
        tasks=[task],
        validate_and_filter=True,
    )
    result2 = cache.load_results(
        models=[
            model2,
        ],
        tasks=[task],
        validate_and_filter=True,
    )
    assert len(result1.model_results[0].task_results[0].scores["test"]) == 2
    assert len(result2.model_results[0].task_results[0].scores["test"]) == 2

    assert np.isnan(result1.model_results[0].task_results[0].get_score())
    assert result2.model_results[0].task_results[0].get_score() == 0.01035


def test_load_experiment_results(tmp_path):
    """Test that results from an experiment can be loaded correctly."""
    model = mteb.get_model("mteb/baseline-random-encoder")
    task = MockRetrievalTask()
    cache = mteb.ResultCache(tmp_path)
    mteb.evaluate(model, task, cache=cache)

    params_1 = {"a": "test"}
    params_2 = {"a": "test", "b": "test2"}
    model = mteb.get_model(
        "mteb/baseline-random-encoder",
        **params_1,
    )
    mteb.evaluate(model, task, cache=cache)

    model = mteb.get_model(
        "mteb/baseline-random-encoder",
        **params_2,
    )
    mteb.evaluate(model, task, cache=cache)

    # load without experiments - should only get the first result
    base_res = cache.load_results()
    assert len(base_res.model_results) == 1
    assert base_res.model_results[0].experiment_name is None

    base_res = cache.load_results(experiment_kwargs=[params_1, params_2])
    assert len(base_res.model_results) == 2

    # load all experiments
    experiment_res = cache.load_results(load_experiments=LoadExperimentEnum.MATCH_NAME)
    assert len(experiment_res.model_results) == 3

    # don't load experiments
    experiment_res = cache.load_results(
        load_experiments=LoadExperimentEnum.NO_EXPERIMENTS
    )
    assert len(experiment_res.model_results) == 1

    # load **only** specific experiment by kwargs
    only_named_experiment_res = cache.load_results(
        load_experiments=LoadExperimentEnum.MATCH_KWARGS,
        experiment_kwargs=params_1,
    )
    assert len(only_named_experiment_res.model_results) == 1
    assert only_named_experiment_res.model_results[0].experiment_name == "a_test"

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta],
        load_experiments=LoadExperimentEnum.MATCH_NAME,
        experiment_kwargs=params_1,
    )
    assert len(model_meta_res.model_results) == 1

    # load specific experiment with model meta filter
    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta.name],
        load_experiments=LoadExperimentEnum.MATCH_NAME,
        experiment_kwargs=params_2,
    )
    assert len(model_meta_res.model_results) == 1
    assert model_meta_res.model_results[0].experiment_name == "a_test__b_test2"

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta],
        load_experiments=LoadExperimentEnum.MATCH_KWARGS,
    )
    assert len(model_meta_res.model_results) == 1

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta],
        load_experiments=LoadExperimentEnum.NO_EXPERIMENTS,
    )
    assert len(model_meta_res.model_results) == 1

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta],
        load_experiments=LoadExperimentEnum.MATCH_KWARGS,
    )
    assert len(model_meta_res.model_results) == 1
    assert model_meta_res.model_results[0].experiment_name == "a_test__b_test2"

    # load experiments with model name filter
    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta.name],
        load_experiments=LoadExperimentEnum.MATCH_NAME,
    )
    assert len(model_meta_res.model_results) == 3

    # load specific experiment with model name filter
    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta.name],
        load_experiments=LoadExperimentEnum.MATCH_KWARGS,
        experiment_kwargs=[model.mteb_model_meta.experiment_kwargs],
    )
    assert len(model_meta_res.model_results) == 1
    assert model_meta_res.model_results[0].experiment_name == "a_test__b_test2"

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta],
        load_experiments=LoadExperimentEnum.MATCH_NAME,
    )
    assert len(model_meta_res.model_results) == 3

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta],
        load_experiments=LoadExperimentEnum.MATCH_KWARGS,
    )
    assert len(model_meta_res.model_results) == 1

    model_meta_res = cache.load_results(
        models=[model.mteb_model_meta.name],
        load_experiments=LoadExperimentEnum.MATCH_KWARGS,
    )
    assert len(model_meta_res.model_results) == 1

    model_meta_res = cache.load_results(
        experiment_kwargs=[model.mteb_model_meta.experiment_kwargs],
    )
    assert len(model_meta_res.model_results) == 1


# Tests for _download_cached_results_from_branch method


class TestDownloadCachedResultsFromBranch:
    """Test the _download_cached_results_from_branch method."""

    @patch("requests.get")
    def test_successful_download(
        self, mock_get, tmp_path, mock_benchmark_json, mock_gzipped_content
    ):
        """Test successful download and decompression, including parent directory creation."""
        cache = ResultCache(cache_path=tmp_path)

        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.headers = {"content-type": "application/gzip"}
        mock_response.content = mock_gzipped_content(mock_benchmark_json)
        mock_get.return_value = mock_response

        # Test basic download
        output_path = tmp_path / "test_cached_results.json"
        result_path = cache._download_cached_results_from_branch(
            output_path=output_path
        )

        assert result_path == output_path
        assert result_path.exists()
        assert result_path.read_text(encoding="utf-8") == mock_benchmark_json
        mock_response.raise_for_status.assert_called_once()

        # Test parent directory creation
        nested_output_path = tmp_path / "nested" / "dir" / "test_cache.json"
        nested_result_path = cache._download_cached_results_from_branch(
            output_path=nested_output_path
        )

        assert nested_result_path.exists()
        assert nested_result_path.read_text(encoding="utf-8") == mock_benchmark_json

    @patch("requests.get")
    def test_file_too_large(self, mock_get, tmp_path):
        """Test that oversized files raise ValueError."""
        cache = ResultCache(cache_path=tmp_path)

        # Create gzipped content that's 51MB (larger than the limit)
        # We create the actual bytes directly rather than compressing,
        # since we're testing the downloaded size check
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.headers = {"content-type": "application/gzip"}
        mock_response.content = b"x" * (51 * 1024 * 1024)  # 51MB of gzipped data
        mock_get.return_value = mock_response

        with pytest.raises(ValueError, match="Downloaded file too large"):
            cache._download_cached_results_from_branch(max_size_mb=50)

    @pytest.mark.parametrize(
        "error_type,exception_class",
        [
            ("timeout", requests.exceptions.Timeout),
            ("connection", requests.exceptions.ConnectionError),
            ("http_404", requests.exceptions.HTTPError),
            ("invalid_gzip", gzip.BadGzipFile),
        ],
    )
    @patch("requests.get")
    def test_error_handling_consolidated(
        self, mock_get, tmp_path, error_type, exception_class
    ):
        """Test various error conditions in a consolidated manner."""
        cache = ResultCache(cache_path=tmp_path)

        if error_type == "timeout":
            mock_get.side_effect = requests.exceptions.Timeout("Request timed out")
        elif error_type == "connection":
            mock_get.side_effect = requests.exceptions.ConnectionError(
                "Connection failed"
            )
        elif error_type == "http_404":
            mock_response = Mock()
            mock_response.status_code = 404
            mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError(
                "404 Not Found"
            )
            mock_get.return_value = mock_response
        elif error_type == "invalid_gzip":
            mock_response = Mock()
            mock_response.status_code = 200
            mock_response.headers = {"content-type": "application/gzip"}
            mock_response.content = b"not gzipped data"
            mock_get.return_value = mock_response

        with pytest.raises(exception_class):
            cache._download_cached_results_from_branch(timeout=30)

    @patch("requests.get")
    def test_default_output_path(
        self, mock_get, tmp_path, mock_benchmark_json, mock_gzipped_content
    ):
        """Test that default output path is {cache_path}/leaderboard/__cached_results.json when none provided."""
        cache = ResultCache(cache_path=tmp_path)

        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.headers = {"content-type": "application/gzip"}
        mock_response.content = mock_gzipped_content(mock_benchmark_json)
        mock_get.return_value = mock_response

        result_path = cache._download_cached_results_from_branch()

        # Default path should be {cache_path}/leaderboard/__cached_results.json
        expected_path = tmp_path / "leaderboard" / "__cached_results.json"
        assert result_path == expected_path
        assert result_path.exists()

    @pytest.mark.parametrize(
        "content_type,should_raise_exception",
        [
            ("application/gzip", False),  # Expected type
            ("text/html", True),  # Unexpected type - should raise exception
            ("", False),  # Empty content-type should not raise exception
        ],
    )
    @patch("requests.get")
    def test_content_type_handling(
        self,
        mock_get,
        tmp_path,
        content_type,
        should_raise_exception,
        mock_benchmark_json,
        mock_gzipped_content,
    ):
        """Test various content-type header scenarios."""
        cache = ResultCache(cache_path=tmp_path)

        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.headers = {"content-type": content_type}
        mock_response.content = mock_gzipped_content(mock_benchmark_json)
        mock_get.return_value = mock_response

        output_path = tmp_path / "test.json"

        if should_raise_exception:
            with pytest.raises(Exception, match="Unexpected content-type"):
                cache._download_cached_results_from_branch(output_path=output_path)
        else:
            result_path = cache._download_cached_results_from_branch(
                output_path=output_path
            )
            assert result_path.exists()
            assert result_path.read_text(encoding="utf-8") == mock_benchmark_json

    @pytest.mark.parametrize(
        "file_size,max_size_mb,should_fail",
        [
            (1024, 1, False),  # Small file - OK
            (1024 * 1024, 1, False),  # At limit - OK
            (51 * 1024 * 1024, 50, True),  # Over limit - Fail
        ],
    )
    @patch("requests.get")
    def test_file_size_validation(
        self,
        mock_get,
        tmp_path,
        file_size,
        max_size_mb,
        should_fail,
        mock_gzipped_content,
    ):
        """Test file size validation with various sizes.

        Note: file_size represents the size of the gzipped (downloaded) content,
        not the uncompressed size, since validation happens on download.
        """
        cache = ResultCache(cache_path=tmp_path)

        # For size validation, we care about the downloaded (gzipped) size
        # So we mock the response.content directly with the desired byte size
        # But we still need valid gzip content for decompression
        if should_fail:
            # For failure cases, just use raw bytes (will fail on decompression but that's after size check)
            mock_response = Mock()
            mock_response.status_code = 200
            mock_response.headers = {"content-type": "application/gzip"}
            mock_response.content = b"x" * file_size
            mock_get.return_value = mock_response

            with pytest.raises(ValueError, match="Downloaded file too large"):
                cache._download_cached_results_from_branch(max_size_mb=max_size_mb)
        else:
            # For success cases, use valid gzipped content
            # Create small content and gzip it
            content = "x" * min(file_size, 1000)  # Keep content small for faster tests
            gzipped = mock_gzipped_content(content)

            mock_response = Mock()
            mock_response.status_code = 200
            mock_response.headers = {"content-type": "application/gzip"}
            mock_response.content = gzipped
            mock_get.return_value = mock_response

            result_path = cache._download_cached_results_from_branch(
                max_size_mb=max_size_mb
            )
            assert result_path.exists()



from __future__ import annotations

import logging
import warnings
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any, Literal, cast, get_args, overload

import mteb
from mteb.abstasks.task_metadata import TaskType
from mteb.similarity_functions import (
    cos_sim,
    dot_score,
    max_sim,
    pairwise_cos_sim,
    pairwise_dot_score,
    pairwise_max_sim,
)
from mteb.types import (
    PromptType,
)

from .model_meta import ScoringFunction

if TYPE_CHECKING:
    from collections.abc import Callable, Sequence

    from torch.utils.data import DataLoader
    from typing_extensions import Unpack

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import (
        Array,
        BatchedInput,
        EncodeKwargs,
    )

    from .model_meta import ModelMeta

logger = logging.getLogger(__name__)


class AbsEncoder(ABC):
    """Base class to indicate that this is a wrapper for a model.

    Also contains some utility functions for wrappers for working with prompts and instructions.

    Attributes:
        model: The model to be wrapped.
        mteb_model_meta: Metadata about the model.
        model_prompts: A dictionary of prompts to be used for encoding sentences.
        instruction_template: A template for formatting instructions. Can be a string with '{instruction}' or
            a callable that takes the instruction and prompt type and returns a formatted instruction.
        prompts_dict: A dictionary of prompts to be used for encoding sentences, overrides model_prompts if provided.
    """

    model: Any
    mteb_model_meta: ModelMeta | None = None
    model_prompts: dict[str, str] | None = None
    instruction_template: str | Callable[[str, PromptType | None], str] | None = None
    prompts_dict: dict[str, str] | None = None

    def get_prompt_name(
        self,
        task_metadata: TaskMetadata,
        prompt_type: PromptType | None,
    ) -> str | None:
        """A wrapper function around the model.encode method that handles the prompt_name argument and standardizes the output to a numpy array.

        The order of priorities for prompt selection are:
            1. Composed prompt of task name + prompt type
            2. Specific task prompt
            3. Composed prompt of task type + prompt type
            4. Specific task type prompt
            5. Specific prompt type

        Args:
            task_metadata: The task name to use for building the encoding prompt
            prompt_type: The prompt type (e.g. "query" | "document") to use for building the encoding prompt

        Returns:
            The name of the prompt to use, or None if no prompt is found.
        """
        return get_prompt_name(self.model_prompts, task_metadata, prompt_type)

    def get_prompt(
        self,
        task_metadata: TaskMetadata,
        prompt_type: PromptType | None,
    ) -> str | None:
        """Get the prompt to be used for encoding sentences.

        Args:
            task_metadata: The metadata of the task.
            prompt_type: The name type of prompt.
        """
        return get_prompt_name(self.model_prompts, task_metadata, prompt_type)

    @staticmethod
    @overload
    def validate_task_to_prompt_name(
        task_to_prompt: dict[str, str] | None,
        raise_for_invalid_keys: Literal[True] = True,
    ) -> dict[str, str] | None: ...

    @staticmethod
    @overload
    def validate_task_to_prompt_name(
        task_to_prompt: dict[str, str] | None,
        raise_for_invalid_keys: Literal[False] = False,
    ) -> tuple[dict[str, str], Sequence[str]] | tuple[None, None]: ...

    @staticmethod
    def validate_task_to_prompt_name(
        task_to_prompt: dict[str, str] | None,
        raise_for_invalid_keys: bool = True,
    ) -> (
        dict[str, str] | tuple[dict[str, str], Sequence[str]] | tuple[None, None] | None
    ):
        """Validates that the keys in task_to_prompt_name map to a known task or prompt type.

        A key is valid if:

        1.  It is a valid task name; or
        2.  It is a valid task type; or
        3.  It is a valid prompt type; or
        4.  It is a compound key of the form "{task_name}-{prompt_type}" where task_name is a valid task type or task
            name and prompt_type is a valid prompt type.

        See the
        [MTEB docs](https://github.com/embeddings-benchmark/mteb/blob/main/docs/usage/usage.md#running-sentencetransformer-model-with-prompts)
        for a complete description of the order or precedence for these keys when running an evaluation.

        Arguments:
            task_to_prompt: The dictionary of prompts.
            raise_for_invalid_keys: If True, raise an error when an invalid key is encountered, otherwise return the
                list of error messages along with a filtered dictionary of prompts with valid keys. Defaults to True
                for backward compatibility.

        Returns:
            - None if `task_to_prompt` is None or empty;
            - Only a dictionary of validated prompts if `raise_for_invalid_keys` is `True`; or
            - A tuple containing the filtered dictionary of valid prompts and the set of error messages for the
                invalid prompts `raise_for_invalid` is `False`
        """
        if not task_to_prompt:
            return None if raise_for_invalid_keys else (None, None)

        task_types = get_args(TaskType)
        prompt_types = [e.value for e in PromptType]
        valid_keys_msg = f"Valid keys are task types [{task_types}], prompt types [{prompt_types}], and task names"
        valid_prompt_type_endings = tuple(
            f"-{prompt_type}" for prompt_type in prompt_types
        )

        invalid_keys: set[str] = set()
        invalid_task_messages: set[str] = set()

        for task_key in task_to_prompt:
            # task_key may be a compound key of the form "{task_name}-{prompt_type}". A task_name may contain a "-"
            # character (this occurs in ~12% of task names), so rsplit is used to separate a valid prompt_type postfix
            # from the unvalidated task_name.
            if task_key.endswith(valid_prompt_type_endings):
                task_name = task_key.rsplit("-", 1)[0]
            else:
                task_name = task_key

            if task_name not in task_types and task_name not in prompt_types:
                try:
                    mteb.get_task(task_name=task_name)
                except KeyError:
                    msg = f"Task name {task_name} is not valid. {valid_keys_msg}"
                    logger.warning(msg)
                    warnings.warn(msg)
                    invalid_task_messages.add(msg)
                    invalid_keys.add(task_key)

        if raise_for_invalid_keys and invalid_task_messages:
            raise KeyError(invalid_task_messages)
        elif raise_for_invalid_keys:
            return task_to_prompt
        else:
            return {
                k: v for k, v in task_to_prompt.items() if k not in invalid_keys
            }, tuple(invalid_task_messages)

    def get_instruction(
        self,
        task_metadata: TaskMetadata,
        prompt_type: PromptType | None,
    ) -> str:
        """Get the instruction/prompt to be used for encoding sentences.

        Args:
            task_metadata: The metadata of the task. Sentence-transformers uses this to
                determine which prompt to use from a specified dictionary.
                The order of priorities for prompt selection are:
                    1. Specific task prompt
                    2. Specific task type prompt
                    3. Specific prompt type
                    4. Default prompt from the task definition
            prompt_type: The name type of prompt.

        Returns:
            The instruction/prompt to be used for encoding sentences.
        """
        prompt = task_metadata.prompt
        if self.prompts_dict and task_metadata.name in self.prompts_dict:
            prompt = self.prompts_dict[task_metadata.name]
        elif self.prompts_dict and task_metadata.type in self.prompts_dict:
            prompt = self.prompts_dict[task_metadata.type]
        elif (
            self.prompts_dict and prompt_type and prompt_type.value in self.prompts_dict
        ):
            prompt = self.prompts_dict[prompt_type.value]

        if isinstance(prompt, dict) and prompt_type:
            if prompt.get(prompt_type.value):
                return prompt[prompt_type.value]
            msg = f"Prompt type '{prompt_type}' not found in task metadata for task '{task_metadata.name}'."
            logger.warning(msg)
            warnings.warn(msg)
            return ""

        if prompt:
            return prompt

        abstask = mteb.get_task(task_name=task_metadata.name)
        return abstask.abstask_prompt

    def format_instruction(
        self, instruction: str, prompt_type: PromptType | None = None
    ) -> str:
        """Format the instruction using the instruction template.

        Args:
            instruction: The instruction to be formatted.
            prompt_type: The name type of prompt.
        """
        if self.instruction_template is None:
            raise ValueError(
                "Attempting to format an instruction without an instruction template."
            )
        if isinstance(self.instruction_template, str):
            if "{instruction}" not in self.instruction_template:
                raise ValueError(
                    "Instruction template must contain the string '{instruction}'."
                )
            return self.instruction_template.format(instruction=instruction)
        return self.instruction_template(instruction, prompt_type)

    def get_task_instruction(
        self,
        task_metadata: TaskMetadata,
        prompt_type: PromptType | None,
    ) -> str:
        """Create the instruction to be used for encoding sentences.

        Args:
            task_metadata: The metadata of the task
            prompt_type: The name type of prompt.

        Returns:
            The instruction to be used for encoding sentences.
        """
        instruction = self.get_instruction(task_metadata, prompt_type)
        if self.instruction_template and len(instruction) > 0:
            return self.format_instruction(instruction, prompt_type)
        return instruction

    def similarity(self, embeddings1: Array, embeddings2: Array) -> Array:
        """Compute the similarity between two collections of embeddings.

        The output will be a matrix with the similarity scores between all embeddings from the first parameter and all
        embeddings from the second parameter. This differs from similarity_pairwise which computes the similarity
        between corresponding pairs of embeddings.

        Read more at: https://www.sbert.net/docs/package_reference/sentence_transformer/SentenceTransformer.html#sentence_transformers.SentenceTransformer.similarity

        Args:
            embeddings1: [num_embeddings_1, embedding_dim] or [embedding_dim]-shaped numpy array or torch tensor.
            embeddings2: [num_embeddings_2, embedding_dim] or [embedding_dim]-shaped numpy array or torch tensor.

        Returns:
            A [num_embeddings_1, num_embeddings_2]-shaped torch tensor with similarity scores.
        """
        if self.mteb_model_meta is None or (
            self.mteb_model_meta is not None
            and self.mteb_model_meta.similarity_fn_name is None
        ):
            if (
                hasattr(self, "model")
                and hasattr(self.model, "similarity")
                and callable(self.model.similarity)
            ):
                arr = self.model.similarity(embeddings1, embeddings2)
                # We assume that the model returns an Array-like object:
                arr = cast("Array", arr)
                return arr
            return cos_sim(embeddings1, embeddings2)
        if self.mteb_model_meta.similarity_fn_name is ScoringFunction.COSINE:
            return cos_sim(embeddings1, embeddings2)
        elif self.mteb_model_meta.similarity_fn_name is ScoringFunction.DOT_PRODUCT:
            return dot_score(embeddings1, embeddings2)
        elif self.mteb_model_meta.similarity_fn_name is ScoringFunction.MAX_SIM:
            return max_sim(embeddings1, embeddings2)
        raise ValueError("Similarity function not specified.")

    def similarity_pairwise(
        self,
        embeddings1: Array,
        embeddings2: Array,
    ) -> Array:
        """Compute the similarity between two collections of embeddings. The output will be a vector with the similarity scores between each pair of embeddings.

        Read more at: https://www.sbert.net/docs/package_reference/sentence_transformer/SentenceTransformer.html#sentence_transformers.SentenceTransformer.similarity_pairwise

        Args:
            embeddings1: [num_embeddings, embedding_dim] or [embedding_dim]-shaped numpy array or torch tensor.
            embeddings2: [num_embeddings, embedding_dim] or [embedding_dim]-shaped numpy array or torch tensor.

        Returns:
            A [num_embeddings]-shaped torch tensor with pairwise similarity scores.
        """
        if self.mteb_model_meta is None or (
            self.mteb_model_meta is not None
            and self.mteb_model_meta.similarity_fn_name is None
        ):
            if (
                hasattr(self, "model")
                and hasattr(self.model, "similarity_pairwise")
                and callable(self.model.similarity_pairwise)
            ):
                arr = self.model.similarity_pairwise(embeddings1, embeddings2)
                # We assume that the model returns an Array-like object:
                arr = cast("Array", arr)
                return arr
            return pairwise_cos_sim(embeddings1, embeddings2)
        if self.mteb_model_meta.similarity_fn_name is ScoringFunction.COSINE:
            return pairwise_cos_sim(embeddings1, embeddings2)
        elif self.mteb_model_meta.similarity_fn_name is ScoringFunction.DOT_PRODUCT:
            return pairwise_dot_score(embeddings1, embeddings2)
        elif self.mteb_model_meta.similarity_fn_name is ScoringFunction.MAX_SIM:
            return pairwise_max_sim(embeddings1, embeddings2)
        raise ValueError("Similarity function not specified.")

    @abstractmethod
    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Unpack[EncodeKwargs],
    ) -> Array:
        """Encodes the given sentences using the encoder.

        Args:
            inputs: Batch of inputs to encode.
            task_metadata: The metadata of the task. Sentence-transformers uses this to
                determine which prompt to use from a specified dictionary.
                The order of priorities for prompt selection are:
                    1. Composed prompt of task name + prompt type
                    2. Specific task prompt
                    3. Composed prompt of task type + prompt type
                    4. Specific task type prompt
                    5. Specific prompt type
            hf_split: Split of current task
            hf_subset: Subset of current task
            prompt_type: The name type of prompt.
            **kwargs: Additional arguments to pass to the encoder.

        Returns:
            The encoded input in a numpy array or torch tensor of the shape (Number of sentences) x (Embedding dimension).
        """
        raise NotImplementedError(
            "The encode method must be implemented in the subclass."
        )


def get_prompt_name(
    model_prompts: dict[str, str] | None,
    task_metadata: TaskMetadata,
    prompt_type: PromptType | None,
) -> str | None:
    """A wrapper function around the model.encode method that handles the prompt_name argument and standardizes the output to a numpy array.

    The order of priorities for prompt selection are:
        1. Composed prompt of task name + prompt type
        2. Specific task prompt
        3. Composed prompt of task type + prompt type
        4. Specific task type prompt
        5. Specific prompt type

    Args:
        model_prompts: A dictionary of prompts to be used for encoding sentences.
        task_metadata: The task name to use for building the encoding prompt
        prompt_type: The prompt type (e.g. "query" | "document") to use for building the encoding prompt

    Returns:
        The name of the prompt to use, or None if no prompt is found.
    """
    if model_prompts is None:
        return None
    task_type = task_metadata.type
    prompt_type_value = prompt_type.value if prompt_type else None
    task_name = task_metadata.name

    if (
        task_name
        and prompt_type
        and f"{task_name}-{prompt_type_value}" in model_prompts
    ):
        return f"{task_name}-{prompt_type_value}"
    if task_name and task_name in model_prompts:
        return task_name
    if (
        task_type
        and prompt_type
        and f"{task_type}-{prompt_type_value}" in model_prompts
    ):
        return f"{task_type}-{prompt_type_value}"
    if task_type and task_type in model_prompts:
        return task_type
    if prompt_type and prompt_type_value in model_prompts:
        return prompt_type_value
    logger.info(
        "No combination of task name and prompt type was found in model prompts."
    )
    return None


def get_prompt(
    model_prompts: dict[str, str] | None,
    task_metadata: TaskMetadata,
    prompt_type: PromptType | None,
) -> str | None:
    """Get the prompt to be used for encoding sentences.

    Args:
        model_prompts: A dictionary of prompts to be used for encoding sentences.
        task_metadata: The metadata of the task.
        prompt_type: The name type of prompt.
    """
    if not model_prompts:
        return None
    prompt_name = get_prompt_name(model_prompts, task_metadata, prompt_type)
    return model_prompts.get(prompt_name) if prompt_name else None



from typing import ClassVar


class LogOnce:
    """Logger wrapper that ensures each unique message is logged only once per logger instance."""

    _seen: ClassVar[dict] = {}  # Class-level cache shared across instances

    def __init__(self, logger):
        self.logger = logger
        # Use logger name as key to have separate caches per logger
        if logger.name not in self._seen:
            self._seen[logger.name] = set()

    def info(self, msg):
        if msg not in self._seen[self.logger.name]:
            self.logger.info(msg)
            self._seen[self.logger.name].add(msg)

    def warning(self, msg):
        if msg not in self._seen[self.logger.name]:
            self.logger.warning(msg)
            self._seen[self.logger.name].add(msg)

    def error(self, msg):
        if msg not in self._seen[self.logger.name]:
            self.logger.error(msg)
            self._seen[self.logger.name].add(msg)

    def debug(self, msg):
        if msg not in self._seen[self.logger.name]:
            self.logger.debug(msg)
            self._seen[self.logger.name].add(msg)



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "fi": ["fin-Latn"],
    "fr": ["fra-Latn"],
    "ru": ["rus-Cyrl"],
    "sv": ["swe-Latn"],
}


class OpusparcusPC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="OpusparcusPC",
        dataset={
            "path": "mteb/OpusparcusPC",
            "revision": "e95831b0fa902edc955f3b60e3e63b349bb7bd02",
        },
        description="Opusparcus is a paraphrase corpus for six European language: German, English, Finnish, French, Russian, and Swedish. The paraphrases consist of subtitles from movies and TV shows.",
        reference="https://gem-benchmark.com/data_cards/opusparcus",
        category="t2t",
        modalities=["text"],
        type="PairClassification",
        eval_splits=["test.full", "validation.full"],
        eval_langs=_LANGUAGES,
        main_score="max_ap",
        date=("2013-01-01", "2015-12-31"),
        domains=["Spoken", "Spoken"],
        task_subtypes=[],
        license="cc-by-nc-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@misc{creutz2018open,
  archiveprefix = {arXiv},
  author = {Mathias Creutz},
  eprint = {1809.06142},
  primaryclass = {cs.CL},
  title = {Open Subtitles Paraphrase Corpus for Six Languages},
  year = {2018},
}
""",
    )



from datasets import Features, load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.retrieval_dataset_loaders import RetrievalSplitData
from mteb.abstasks.task_metadata import TaskMetadata
from mteb.types._encoder_io import VideoInputItem


class MSRVTTV2T(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MSRVTTV2T",
        description="MSRVTT",
        dataset={
            "path": "mteb/MSR-VTT",
            "revision": "4661603cee25c1fd370e5478a2953203cf37155b",
        },
        type="Retrieval",
        eval_langs=["eng-Latn"],
        eval_splits=["test"],
        main_score="ndcg_at_10",
        reference=None,
        category="v2t",
        modalities=["video", "text"],
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        """Load the MSRVTT dataset."""
        if self.data_loaded:
            return
        dataset = load_dataset(
            self.metadata.dataset["path"],
            revision=self.metadata.dataset["revision"],
            split=self.metadata.eval_splits[0],
        )
        dataset = dataset.add_column("id", [str(i) for i in range(len(dataset))])
        query = dataset.select_columns(["id", "video", "audio"])

        def _combine_modalities(example: dict) -> dict:
            example["video"] = VideoInputItem(
                frames=example["video"],
                audio=example.pop("audio"),
            )

            return example

        query_features = query.features
        query = query.map(
            _combine_modalities,
            features=Features(
                {
                    "id": query_features["id"],
                    "video": {
                        "frames": query_features["video"],
                        "audio": query_features["audio"],
                    },
                }
            ),
        )
        corpus = dataset.select_columns(["id", "caption"]).rename_column(
            "caption", "text"
        )
        qrels = {}
        for i in range(len(dataset)):
            key = str(i)
            if key not in qrels:
                qrels[key] = {}
            qrels[key][key] = 1
        self.dataset["default"]["test"] = RetrievalSplitData(
            queries=query, corpus=corpus, relevant_docs=qrels, top_ranked=None
        )
        self.data_loaded = True



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class GlobeV3AgeClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="GLOBEV3Age",
        description="Age classification from the GLOBE v3 dataset (sampled and enhanced from CommonVoice dataset for TTS purpose). This dataset is a stratified and downsampled version of the original dataset, containing about 535 hours of speech data across 164 accents. We use the age column as the target label for audio classification.",
        reference="https://huggingface.co/datasets/MushanW/GLOBE_V3",
        dataset={
            "path": "mteb/globe-v3-age-mini",
            "revision": "f7399f4b836508a178c0913868e82462b4a8919b",
        },
        type="AudioClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2025-05-26", "2025-05-26"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Age Classification"],
        license="cc0-1.0",
        annotations_creators="automatic",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{wang2024globe,
  archiveprefix = {arXiv},
  author = {Wenbin Wang and Yang Song and Sanjay Jha},
  eprint = {2406.14875},
  title = {GLOBE: A High-quality English Corpus with Global Accents for Zero-shot Speaker Adaptive Text-to-Speech},
  year = {2024},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "predicted_age"



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class RenderedSST2(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="RenderedSST2",
        description="RenderedSST2.",
        reference="https://huggingface.co/datasets/clip-benchmark/wds_renderedsst2",
        dataset={
            "path": "mteb/wds_renderedsst2",
            "revision": "c10537bc389e9741a27f7b14767bd42f2b77476b",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2016-01-01", "2016-12-31"),
        domains=["Reviews"],
        task_subtypes=[],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""""",
    )

    # Override default column names in the subclass
    input_column_name: str = "png"
    label_column_name: str = "cls"

    def get_candidate_labels(self) -> list[str]:
        return ["a negative review of a movie", "a positive review of a movie"]



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class NSynth(AbsTaskClassification):
    metadata = TaskMetadata(
        name="NSynth",
        description="Instrument Source Classification: one of acoustic, electronic, or synthetic.",
        reference="https://huggingface.co/datasets/anime-sh/NSYNTH_PITCH_HEAR",
        dataset={
            "path": "mteb/nsynth-mini",
            "revision": "e32dfe9b65e121e64229a821fe1ff177e8962635",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="accuracy",
        date=("2025-03-06", "2025-03-06"),
        domains=["Music"],
        task_subtypes=["Instrument Source Classification"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{engel2017neuralaudiosynthesismusical,
  archiveprefix = {arXiv},
  author = {Jesse Engel and Cinjon Resnick and Adam Roberts and Sander Dieleman and Douglas Eck and Karen Simonyan and Mohammad Norouzi},
  eprint = {1704.01279},
  primaryclass = {cs.LG},
  title = {Neural Audio Synthesis of Musical Notes with WaveNet Autoencoders},
  url = {https://arxiv.org/abs/1704.01279},
  year = {2017},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "instrument_source"



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class TwitterHjerneRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="TwitterHjerneRetrieval",
        dataset={
            "path": "mteb/TwitterHjerneRetrieval",
            "revision": "97ad55673cf9746f8e4b3aaa92b1bb92d82e52db",
        },
        description="Danish question asked on Twitter with the Hashtag #Twitterhjerne ('Twitter brain') and their corresponding answer.",
        reference="https://huggingface.co/datasets/sorenmulli/da-hashtag-twitterhjerne",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["dan-Latn"],
        main_score="ndcg_at_10",
        date=("2006-01-01", "2024-12-31"),  # best guess
        domains=["Social", "Written"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{holm2024gllms,
  author = {Holm, Soren Vejlgaard},
  title = {Are GLLMs Danoliterate? Benchmarking Generative NLP in Danish},
  year = {2024},
}
""",
        prompt={"query": "Retrieve answers to questions asked in Danish tweets"},
        task_subtypes=["Question answering"],
    )



from .polish_clustering import (
    EightTagsClustering,
    EightTagsClusteringFast,
    PlscClusteringP2P,
    PlscClusteringP2PFast,
    PlscClusteringS2S,
    PlscClusteringS2SFast,
)

__all__ = [
    "EightTagsClustering",
    "EightTagsClusteringFast",
    "PlscClusteringP2P",
    "PlscClusteringP2PFast",
    "PlscClusteringS2S",
    "PlscClusteringS2SFast",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class LibriTTSA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="LibriTTSA2TRetrieval",
        description=(
            "Given audiobook speech segments from the multi‑speaker LibriTTS corpus, "
            "retrieve the correct text transcription. LibriTTS is a 585‑hour, 24 kHz, "
            "multi‑speaker English TTS corpus derived from LibriVox (audio) and Project Gutenberg (text)."
        ),
        reference="https://www.openslr.org/60/",
        dataset={
            "path": "mteb/LibriTTS_a2t",
            "revision": "dbf3f317f96023e103b98548a3b99cfa919afb56",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2019-11-01", "2019-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Transcription Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{zen2019librittscorpusderivedlibrispeech,
  archiveprefix = {arXiv},
  author = {Heiga Zen and Viet Dang and Rob Clark and Yu Zhang and Ron J. Weiss and Ye Jia and Zhifeng Chen and Yonghui Wu},
  eprint = {1904.02882},
  primaryclass = {cs.SD},
  title = {LibriTTS: A Corpus Derived from LibriSpeech for Text-to-Speech},
  url = {https://arxiv.org/abs/1904.02882},
  year = {2019},
}
""",
    )


class LibriTTST2ARetrieval(AbsTaskRetrieval):
    """Retrieval of speech segments given text queries, on the LibriTTS dataset."""

    metadata = TaskMetadata(
        name="LibriTTST2ARetrieval",
        description=(
            "Given an English text transcription, retrieve its corresponding audiobook "
            "speech segment from the multi‑speaker LibriTTS corpus. LibriTTS is a 585‑hour, 24 kHz, "
            "multi‑speaker English TTS corpus derived from LibriVox and Project Gutenberg."
        ),
        reference="https://www.openslr.org/60/",
        dataset={
            "path": "mteb/LibriTTS_t2a",
            "revision": "d6fc5fdcdc0940892c84461e8cfcb908baa717e4",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2019-11-01", "2019-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Transcription Retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{zen2019librittscorpusderivedlibrispeech,
  archiveprefix = {arXiv},
  author = {Heiga Zen and Viet Dang and Rob Clark and Yu Zhang and Ron J. Weiss and Ye Jia and Zhifeng Chen and Yonghui Wu},
  eprint = {1904.02882},
  primaryclass = {cs.SD},
  title = {LibriTTS: A Corpus Derived from LibriSpeech for Text-to-Speech},
  url = {https://arxiv.org/abs/1904.02882},
  year = {2019},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class UrduRomanSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="UrduRomanSentimentClassification",
        description="The Roman Urdu dataset is a data corpus comprising of more than 20000 records tagged for sentiment (Positive, Negative, Neutral)",
        reference="https://archive.ics.uci.edu/dataset/458/roman+urdu+data+set",
        dataset={
            "path": "mteb/UrduRomanSentimentClassification",
            "revision": "905c1121c002c4b9adc4ebc5faaf4d6f50d1b1ee",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2018-01-01", "2018-08-28"),
        eval_splits=["train"],
        eval_langs=["urd-Latn"],
        main_score="f1",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{misc_roman_urdu_data_set_458,
  author = {Sharf,Zareen},
  howpublished = {UCI Machine Learning Repository},
  note = {{DOI}: https://doi.org/10.24432/C58325},
  title = {{Roman Urdu Data Set}},
  year = {2018},
}
""",
        superseded_by="UrduRomanSentimentClassification.v2",
    )


class UrduRomanSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="UrduRomanSentimentClassification.v2",
        description="The Roman Urdu dataset is a data corpus comprising of more than 20000 records tagged for sentiment (Positive, Negative, Neutral) This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://archive.ics.uci.edu/dataset/458/roman+urdu+data+set",
        dataset={
            "path": "mteb/urdu_roman_sentiment",
            "revision": "fe3ea6b93097e7a2eb1356ad3665fd01667ac6be",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2018-01-01", "2018-08-28"),
        eval_splits=["test"],
        eval_langs=["urd-Latn"],
        main_score="f1",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{misc_roman_urdu_data_set_458,
  author = {Sharf,Zareen},
  howpublished = {UCI Machine Learning Repository},
  note = {{DOI}: https://doi.org/10.24432/C58325},
  title = {{Roman Urdu Data Set}},
  year = {2018},
}
""",
        adapted_from=["UrduRomanSentimentClassification"],
    )



from .angry_tweets_classification import (
    AngryTweetsClassification,
    AngryTweetsClassificationV2,
)
from .danish_political_comments_classification import (
    DanishPoliticalCommentsClassification,
    DanishPoliticalCommentsClassificationV2,
)
from .ddisco_cohesion_classification import (
    DdiscoCohesionClassification,
    DdiscoCohesionClassificationV2,
)
from .dk_hate_classification import DKHateClassification, DKHateClassificationV2
from .lcc_sentiment_classification import LccSentimentClassification

__all__ = [
    "AngryTweetsClassification",
    "AngryTweetsClassificationV2",
    "DKHateClassification",
    "DKHateClassificationV2",
    "DanishPoliticalCommentsClassification",
    "DanishPoliticalCommentsClassificationV2",
    "DdiscoCohesionClassification",
    "DdiscoCohesionClassificationV2",
    "LccSentimentClassification",
]



from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import (
    SentenceTransformerEncoderWrapper,
)

nb_sbert = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[arg-type]
    name="NbAiLab/nb-sbert-base",
    model_type=["dense"],
    languages=["nno-Latn", "nob-Latn", "swe-Latn", "dan-Latn"],
    open_weights=True,
    revision="b95656350a076aeafd2d23763660f80655408cc6",
    release_date="2022-11-23",
    n_parameters=177853440,
    n_embedding_parameters=91_812_096,
    memory_usage_mb=678,
    embed_dim=4096,
    license="apache-2.0",
    max_tokens=75,
    reference="https://huggingface.co/NbAiLab/nb-sbert-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/NbAiLab/mnli-norwegian",
    training_datasets=set(),
)

nb_bert_large = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[arg-type]
    name="NbAiLab/nb-bert-large",
    model_type=["dense"],
    languages=["nno-Latn", "nob-Latn"],
    open_weights=True,
    revision="f9d0fc184adab4dc354d85e1854b7634540d7550",
    release_date="2021-04-29",
    n_parameters=355087360,
    n_embedding_parameters=51_200_000,
    memory_usage_mb=1359,
    embed_dim=1024,
    license="cc-by-4.0",
    max_tokens=512,
    reference="https://huggingface.co/NbAiLab/nb-bert-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/NbAiLab/nb-bert-large#training-data",
    training_datasets=set(),
)

nb_bert_base = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[arg-type]
    name="NbAiLab/nb-bert-base",
    model_type=["dense"],
    languages=["nno-Latn", "nob-Latn"],
    open_weights=True,
    revision="9417c3f62a3adc99f17ff92bff446f35d011f994",
    release_date="2021-01-13",
    n_parameters=177853440,
    n_embedding_parameters=91_812_096,
    memory_usage_mb=681,
    embed_dim=768,
    license="cc-by-4.0",
    max_tokens=512,
    reference="https://huggingface.co/NbAiLab/nb-bert-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/NbAiLab/nb-bert-base#training-data",
    training_datasets=set(),
)



from mteb.models import ModelMeta
from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.types import PromptType

REASONIR_CITATION = """@article{shao2025reasonir,
      title={ReasonIR: Training Retrievers for Reasoning Tasks},
      author={Rulin Shao and Rui Qiao and Varsha Kishore and Niklas Muennighoff and Xi Victoria Lin and Daniela Rus and Bryan Kian Hsiang Low and Sewon Min and Wen-tau Yih and Pang Wei Koh and Luke Zettlemoyer},
      year={2025},
      journal={arXiv preprint arXiv:2504.20595},
      url={https://arxiv.org/abs/2504.20595},
}"""


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    return (
        # https://github.com/facebookresearch/ReasonIR/blob/0aac96269e455965949df16520fab72da68ffc22/evaluation/bright/configs/reasonir/economics.json#L3
        f"<|user|>\n{instruction}\n<|embed|>\n"
        if (prompt_type is None or prompt_type == PromptType.query) and instruction
        else "<|embed|>\n"
    )


REASONIR_TRAINING_DATA = {
    # source, section D: https://arxiv.org/pdf/2504.20595
    "MSMARCO",
    "NQ",
    "FEVER",
    "HotpotQA",
    "MIRACLRetrieval",
    "MrTidyRetrieval",
    "T2Reranking",
    "DuRetrieval",
    "QuoraRetrieval",
}
_prompts_dict = {
    "BrightBiologyRetrieval": {
        "query": "Given a Biology post, retrieve relevant passages that help answer the post"
    },
    "BrightEarthScienceRetrieval": {
        "query": "Given a Earth Science post, retrieve relevant passages that help answer the post"
    },
    "BrightEconomicsRetrieval": {
        "query": "Given a Economics post, retrieve relevant passages that help answer the post"
    },
    "BrightPsychologyRetrieval": {
        "query": "Given a Psychology post, retrieve relevant passages that help answer the post"
    },
    "BrightRoboticsRetrieval": {
        "query": "Given a Robotics post, retrieve relevant passages that help answer the post"
    },
    "BrightStackoverflowRetrieval": {
        "query": "Given a Stackoverflow post, retrieve relevant passages that help answer the post"
    },
    "BrightSustainableLivingRetrieval": {
        "query": "Given a Sustainable Living post, retrieve relevant passages that help answer the post"
    },
    "BrightPonyRetrieval": {
        "query": "Given a Pony question, retrieve relevant passages that help answer the question"
    },
    "BrightLeetcodeRetrieval": {
        "query": "Given a coding problem, retrieve relevant examples that help answer the problem",
    },
    "BrightAopsRetrieval": {
        "query": "Given a Math problem, retrieve relevant examples that help answer the problem"
    },
    "BrightTheoremQATheoremsRetrieval": {
        "query": "Given a Math problem, retrieve relevant theorems that help answer the problem",
    },
    "BrightTheoremQAQuestionsRetrieval": {
        "query": "Given a Math problem, retrieve relevant examples that help answer the problem",
    },
    "BrightBiologyLongRetrieval": {
        "query": "Given a Biology post, retrieve relevant documents that help answer the post"
    },
    "BrightEarthScienceLongRetrieval": {
        "query": "Given a Earth Science post, retrieve relevant documents that help answer the post"
    },
    "BrightEconomicsLongRetrieval": {
        "query": "Given a Economics post, retrieve relevant documents that help answer the post"
    },
    "BrightPsychologyLongRetrieval": {
        "query": "Given a Psychology post, retrieve relevant documents that help answer the post"
    },
    "BrightRoboticsLongRetrieval": {
        "query": "Given a Robotics post, retrieve relevant documents that help answer the post"
    },
    "BrightStackoverflowLongRetrieval": {
        "query": "Given a Stackoverflow post, retrieve relevant documents that help answer the post"
    },
    "BrightSustainableLivingLongRetrieval": {
        "query": "Given a Sustainable Living post, retrieve relevant documents that help answer the post"
    },
    "BrightPonyLongRetrieval": {
        "query": "Given a Pony question, retrieve relevant documents that help answer the question"
    },
}


ReasonIR_8B = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        trust_remote_code=True,
        prompts_dict=_prompts_dict,
    ),
    name="ReasonIR/ReasonIR-8B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="c3d0690370ff4a8c3d3882d8dfa85c43650034fa",
    release_date="2025-04-29",
    n_parameters=7504924672,
    n_embedding_parameters=525336576,
    memory_usage_mb=None,
    embed_dim=4096,
    license="cc-by-nc-4.0",
    max_tokens=131072,
    reference="https://huggingface.co/ReasonIR/ReasonIR-8B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    training_datasets=REASONIR_TRAINING_DATA,
    public_training_code="https://github.com/facebookresearch/ReasonIR/tree/main/training",
    public_training_data="https://huggingface.co/datasets/reasonir/reasonir-data",
    citation=REASONIR_CITATION,
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class AmazonCounterfactualClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="AmazonCounterfactualClassification",
        dataset={
            "path": "mteb/amazon_counterfactual",
            "revision": "1f7e6a9d6fa6e64c53d146e428565640410c0df1",
        },
        description=(
            "A collection of Amazon customer reviews annotated for counterfactual detection pair classification."
        ),
        reference="https://arxiv.org/abs/2104.06893",
        category="t2c",
        modalities=["text"],
        type="Classification",
        eval_splits=["validation", "test"],
        eval_langs={
            "en-ext": ["eng-Latn"],
            "en": ["eng-Latn"],
            "de": ["deu-Latn"],
            "ja": ["jpn-Jpan"],
        },
        main_score="accuracy",
        date=(
            "2018-01-01",
            "2021-12-31",
        ),  # Estimated range for the collection of Amazon reviews
        domains=["Reviews", "Written"],
        task_subtypes=["Counterfactual Detection"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{oneill-etal-2021-wish,
  address = {Online and Punta Cana, Dominican Republic},
  author = {O{'}Neill, James  and
Rozenshtein, Polina  and
Kiryo, Ryuichi  and
Kubota, Motoko  and
Bollegala, Danushka},
  booktitle = {Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing},
  doi = {10.18653/v1/2021.emnlp-main.568},
  editor = {Moens, Marie-Francine  and
Huang, Xuanjing  and
Specia, Lucia  and
Yih, Scott Wen-tau},
  month = nov,
  pages = {7092--7108},
  publisher = {Association for Computational Linguistics},
  title = {{I} Wish {I} Would Have Loved This One, But {I} Didn{'}t {--} A Multilingual Dataset for Counterfactual Detection in Product Review},
  url = {https://aclanthology.org/2021.emnlp-main.568},
  year = {2021},
}
""",
        prompt="Classify a given Amazon customer review text as either counterfactual or not-counterfactual",
    )

    samples_per_label = 32



import datasets

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class NarrativeQARetrieval(AbsTaskRetrieval):
    _EVAL_SPLIT = "test"

    metadata = TaskMetadata(
        name="NarrativeQARetrieval",
        dataset={
            "path": "deepmind/narrativeqa",
            "revision": "2e643e7363944af1c33a652d1c87320d0871c4e4",
        },
        reference="https://metatext.io/datasets/narrativeqa",
        description=(
            "NarrativeQA is a dataset for the task of question answering on long narratives. It consists of "
            + "realistic QA instances collected from literature (fiction and non-fiction) and movie scripts. "
        ),
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2017-01-01", "2017-12-31"),
        domains=["Fiction", "Non-fiction", "Written"],
        task_subtypes=["Question answering"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{kocisky-etal-2018-narrativeqa,
  address = {Cambridge, MA},
  author = {Ko{\v{c}}isk{\'y}, Tom{\'a}{\v{s}}  and
Schwarz, Jonathan  and
Blunsom, Phil  and
Dyer, Chris  and
Hermann, Karl Moritz  and
Melis, G{\'a}bor  and
Grefenstette, Edward},
  doi = {10.1162/tacl_a_00023},
  editor = {Lee, Lillian  and
Johnson, Mark  and
Toutanova, Kristina  and
Roark, Brian},
  journal = {Transactions of the Association for Computational Linguistics},
  pages = {317--328},
  publisher = {MIT Press},
  title = {The {N}arrative{QA} Reading Comprehension Challenge},
  url = {https://aclanthology.org/Q18-1023},
  volume = {6},
  year = {2018},
}
""",
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return

        data = datasets.load_dataset(
            split=self._EVAL_SPLIT,
            **self.metadata.dataset,
        )
        self.queries = {
            self._EVAL_SPLIT: {
                str(i): row["question"]["text"] for i, row in enumerate(data)
            }
        }
        self.corpus = {
            self._EVAL_SPLIT: {
                str(row["document"]["id"]): {"text": row["document"]["text"]}
                for row in data
            }
        }
        self.relevant_docs = {
            self._EVAL_SPLIT: {
                str(i): {row["document"]["id"]: 1} for i, row in enumerate(data)
            }
        }

        self.data_loaded = True



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class Imagenet1kClassification(AbsTaskClassification):
    input_column_name: str = "jpg"
    label_column_name: str = "cls"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="Imagenet1k",
        description="ImageNet, a large-scale ontology of images built upon the backbone of the WordNet structure.",
        reference="https://ieeexplore.ieee.org/document/5206848",
        dataset={
            "path": "mteb/wds_imagenet1k",
            "revision": "b54c9af96641d8a2cba5d1144629c724de94ef76",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2010-01-01",
            "2012-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Scene"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{deng2009imagenet,
  author = {Deng, Jia and Dong, Wei and Socher, Richard and Li, Li-Jia and Kai Li and Li Fei-Fei},
  booktitle = {2009 IEEE Conference on Computer Vision and Pattern Recognition},
  doi = {10.1109/CVPR.2009.5206848},
  keywords = {Large-scale systems;Image databases;Explosions;Internet;Robustness;Information retrieval;Image retrieval;Multimedia databases;Ontologies;Spine},
  number = {},
  pages = {248-255},
  title = {ImageNet: A large-scale hierarchical image database},
  volume = {},
  year = {2009},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class NordicLangClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="NordicLangClassification",
        description="A dataset for Nordic language identification.",
        reference="https://aclanthology.org/2021.vardial-1.8/",
        dataset={
            "path": "mteb/NordicLangClassification",
            "revision": "425d7e9de276902b73d5359ecd4811f3b80cc7c0",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=[
            "nob-Latn",
            "nno-Latn",
            "dan-Latn",
            "swe-Latn",
            "isl-Latn",
            "fao-Latn",
        ],
        main_score="accuracy",
        date=("2020-01-01", "2021-12-31"),  # best guess, year of publication
        domains=["Encyclopaedic"],
        task_subtypes=["Language identification"],
        license="cc-by-sa-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{haas-derczynski-2021-discriminating,
  address = {Kiyv, Ukraine},
  author = {Haas, Ren{\'e}  and
Derczynski, Leon},
  booktitle = {Proceedings of the Eighth Workshop on NLP for Similar Languages, Varieties and Dialects},
  editor = {Zampieri, Marcos  and
Nakov, Preslav  and
Ljube{\v{s}}i{\'c}, Nikola  and
Tiedemann, J{\"o}rg  and
Scherrer, Yves  and
Jauhiainen, Tommi},
  month = apr,
  pages = {67--75},
  publisher = {Association for Computational Linguistics},
  title = {Discriminating Between Similar {N}ordic Languages},
  url = {https://aclanthology.org/2021.vardial-1.8},
  year = {2021},
}
""",
        prompt="Classify texts based on language",
    )

    samples_per_label = 32



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class DBpediaClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DBpediaClassification",
        description="DBpedia14 is a dataset of English texts from Wikipedia articles, categorized into 14 non-overlapping classes based on their DBpedia ontology.",
        reference="https://arxiv.org/abs/1509.01626",
        dataset={
            "path": "mteb/DBpediaClassification",
            "revision": "da1478c37d6ac46d5ca45aaf2a156020e2e9bfea",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2022-01-25", "2022-01-25"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-sa-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{NIPS2015_250cf8b5,
  author = {Zhang, Xiang and Zhao, Junbo and LeCun, Yann},
  booktitle = {Advances in Neural Information Processing Systems},
  editor = {C. Cortes and N. Lawrence and D. Lee and M. Sugiyama and R. Garnett},
  pages = {},
  publisher = {Curran Associates, Inc.},
  title = {Character-level Convolutional Networks for Text Classification},
  url = {https://proceedings.neurips.cc/paper_files/paper/2015/file/250cf8b51c773f3f8dc8b4be867a9a02-Paper.pdf},
  volume = {28},
  year = {2015},
}
""",
        superseded_by="DBpediaClassification.v2",
    )


class DBpediaClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DBpediaClassification.v2",
        description="DBpedia14 is a dataset of English texts from Wikipedia articles, categorized into 14 non-overlapping classes based on their DBpedia ontology. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://arxiv.org/abs/1509.01626",
        dataset={
            "path": "mteb/d_bpedia",
            "revision": "e45aab5cbb44baba43d8a0640d809d2aa0a0a770",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2022-01-25", "2022-01-25"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-sa-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{NIPS2015_250cf8b5,
  author = {Zhang, Xiang and Zhao, Junbo and LeCun, Yann},
  booktitle = {Advances in Neural Information Processing Systems},
  editor = {C. Cortes and N. Lawrence and D. Lee and M. Sugiyama and R. Garnett},
  pages = {},
  publisher = {Curran Associates, Inc.},
  title = {Character-level Convolutional Networks for Text Classification},
  url = {https://proceedings.neurips.cc/paper_files/paper/2015/file/250cf8b51c773f3f8dc8b4be867a9a02-Paper.pdf},
  volume = {28},
  year = {2015},
}
""",
        adapted_from=["DBpediaClassification"],
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["train", "test"]
        )



from .ifir_aila_retrieval import IFIRAila
from .ifir_cds_retrieval import IFIRCds
from .ifir_fi_qa_retrieval import IFIRFiQA
from .ifir_fire_retrieval import IFIRFire
from .ifir_pm_retrieval import IFIRPm
from .ifir_scifact_retrieval import IFIRScifact
from .ifirnf_corpus_retrieval import IFIRNFCorpus
from .instruct_ir import InstructIR

__all__ = [
    "IFIRAila",
    "IFIRCds",
    "IFIRFiQA",
    "IFIRFire",
    "IFIRNFCorpus",
    "IFIRPm",
    "IFIRScifact",
    "InstructIR",
]



from mteb.models import ModelMeta, sentence_transformers_loader

kowshik24_bangla_embedding_model = ModelMeta(
    loader=sentence_transformers_loader,
    name="Kowshik24/bangla-sentence-transformer-ft-matryoshka-paraphrase-multilingual-mpnet-base-v2",
    model_type=["dense"],
    languages=["ben-Beng"],  # Bengali using Bengali script
    open_weights=True,
    revision="6689c21e69be5950596bad084457cbaa138728d8",
    release_date="2025-11-10",
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=128,
    reference="https://huggingface.co/Kowshik24/bangla-sentence-transformer-ft-matryoshka-paraphrase-multilingual-mpnet-base-v2",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code="https://github.com/kowshik24/Bangla-Embedding",
    public_training_data="https://huggingface.co/datasets/sartajekram/BanglaRQA",
    training_datasets=set(),
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FilipinoShopeeReviewsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FilipinoShopeeReviewsClassification",
        description="The Shopee reviews tl 15 dataset is constructed by randomly taking 2100 training samples and 450 samples for testing and validation for each review star from 1 to 5. In total, there are 10500 training samples and 2250 each in validation and testing samples.",
        reference="https://uijrt.com/articles/v4/i8/UIJRTV4I80009.pdf",
        dataset={
            "path": "mteb/FilipinoShopeeReviewsClassification",
            "revision": "c88a93f50ccbb36878cde3362cefd65e43bd6b75",
        },
        type="Classification",
        task_subtypes=["Sentiment/Hate speech"],
        category="t2c",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["fil-Latn"],
        domains=["Social", "Written"],
        license="mpl-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        date=("2022-05-13", "2023-05-13"),
        main_score="accuracy",
        bibtex_citation=r"""
@article{riegoenhancement,
  author = {Riego, Neil Christian R. and Villarba, Danny Bell and Sison, Ariel Antwaun Rolando C. and Pineda, Fernandez C. and Lagunzad, Herminiño C.},
  issue = {08},
  journal = {United International Journal for Research & Technology},
  pages = {72--82},
  title = {Enhancement to Low-Resource Text Classification via Sequential Transfer Learning},
  volume = {04},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class CzechSoMeSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="CzechSoMeSentimentClassification",
        description="User comments on Facebook",
        reference="https://aclanthology.org/W13-1609/",
        dataset={
            "path": "fewshot-goes-multilingual/cs_facebook-comments",
            "revision": "6ced1d87a030915822b087bf539e6d5c658f1988",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ces-Latn"],
        main_score="accuracy",
        date=("2013-01-01", "2013-06-01"),
        dialect=[],
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{habernal-etal-2013-sentiment,
  address = {Atlanta, Georgia},
  author = {Habernal, Ivan  and
Pt{\'a}{\v{c}}ek, Tom{\'a}{\v{s}}  and
Steinberger, Josef},
  booktitle = {Proceedings of the 4th Workshop on Computational Approaches to Subjectivity, Sentiment and Social Media Analysis},
  editor = {Balahur, Alexandra  and
van der Goot, Erik  and
Montoyo, Andres},
  month = jun,
  pages = {65--74},
  publisher = {Association for Computational Linguistics},
  title = {Sentiment Analysis in {C}zech Social Media Using Supervised Machine Learning},
  url = {https://aclanthology.org/W13-1609},
  year = {2013},
}
""",
        superseded_by="CzechSoMeSentimentClassification.v2",
    )
    samples_per_label = 16

    def dataset_transform(self, num_proc: int | None = None, **kwargs) -> None:
        self.dataset = self.dataset.rename_columns(
            {"comment": "text", "sentiment_int": "label"}
        )


class CzechSoMeSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="CzechSoMeSentimentClassification.v2",
        description="User comments on Facebook This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://aclanthology.org/W13-1609/",
        dataset={
            "path": "mteb/czech_so_me_sentiment",
            "revision": "a12152e40ff9857bf3c83694528f40ec5c02aafc",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ces-Latn"],
        main_score="accuracy",
        date=("2013-01-01", "2013-06-01"),
        dialect=[],
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{habernal-etal-2013-sentiment,
  address = {Atlanta, Georgia},
  author = {Habernal, Ivan  and
Pt{\'a}{\v{c}}ek, Tom{\'a}{\v{s}}  and
Steinberger, Josef},
  booktitle = {Proceedings of the 4th Workshop on Computational Approaches to Subjectivity, Sentiment and Social Media Analysis},
  editor = {Balahur, Alexandra  and
van der Goot, Erik  and
Montoyo, Andres},
  month = jun,
  pages = {65--74},
  publisher = {Association for Computational Linguistics},
  title = {Sentiment Analysis in {C}zech Social Media Using Supervised Machine Learning},
  url = {https://aclanthology.org/W13-1609},
  year = {2013},
}
""",
        adapted_from=["CzechSoMeSentimentClassification"],
    )
    samples_per_label = 16



import difflib
import re
from collections import defaultdict
from itertools import combinations

import pytest

import mteb
from mteb import get_task, get_tasks
from mteb.abstasks.abstask import AbsTask
from mteb.abstasks.task_metadata import TaskType
from mteb.get_tasks import MTEBTasks
from mteb.types import Modalities


@pytest.mark.parametrize(
    "task_name", ["BornholmBitextMining", "CQADupstackRetrieval", "Birdsnap"]
)
@pytest.mark.parametrize("eval_splits", [["test"], None])
def test_get_task(
    task_name: str,
    eval_splits: list[str] | None,
):
    task = get_task(
        task_name,
        eval_splits=eval_splits,
    )
    assert isinstance(task, AbsTask)
    assert task.metadata.name == task_name
    if eval_splits:
        for split in task.eval_splits:
            assert split in eval_splits
    else:
        assert task.eval_splits == task.metadata.eval_splits


def test_get_tasks_filtering():
    """Tests that get_tasks filters tasks for languages within the task, i.e. that a multilingual task returns only relevant subtasks for the
    specified languages
    """
    tasks = get_tasks(languages=["eng"])

    for task in tasks:
        if (
            task.metadata.is_multilingual
            and task.metadata.name != "STS17MultilingualVisualSTSEng"
        ):
            assert isinstance(task.metadata.eval_langs, dict)

            for hf_subset in task.hf_subsets:
                assert "eng-Latn" in task.metadata.eval_langs[hf_subset], (
                    f"{task.metadata.name}"
                )


@pytest.mark.parametrize("script", [["Cyrl"], None])
@pytest.mark.parametrize("task_types", [["Classification"], ["Clustering"], None])
@pytest.mark.parametrize("modalities", [["text"], ["image"], None])
def test_mteb_mteb_tasks(
    script: list[str],
    task_types: list[TaskType] | None,
    modalities: list[Modalities] | None,
):
    tasks = mteb.get_tasks(script=script, task_types=task_types, modalities=modalities)
    assert isinstance(tasks, MTEBTasks)
    langs = tasks.languages
    for t in tasks:
        assert len(langs.intersection(t.languages)) > 0

    # check for header of a table
    n_langs = len(tasks)
    assert len(tasks.to_markdown().split("\n")) - 3 == n_langs


@pytest.mark.parametrize("modalities", [["text"], ["image"], ["text", "image"]])
def test_get_tasks_with_exclusive_modality_filter(modalities):
    """Test exclusive_modality_filter with actual tasks (if available)"""
    text_tasks_exclusive = get_tasks(
        modalities=modalities, exclusive_modality_filter=True
    )
    for task in text_tasks_exclusive:
        assert set(task.modalities) == set(modalities)


def test_get_tasks_privacy_filtering():
    """Test that get_tasks correctly filters by privacy status"""
    # By default, should only return public datasets (exclude_private=True)
    public_tasks = get_tasks()

    # Should include private datasets when explicitly requested
    all_tasks = get_tasks(exclude_private=False)

    # All tasks should contain at least as many or more tasks than public tasks
    assert len(all_tasks) >= len(public_tasks)

    # All returned tasks should be public when exclude_private=True
    for task in public_tasks:
        assert (
            task.metadata.is_public is not False
        )  # None or True are both considered public


def _parse_bibtex_entries(bibtex_str: str) -> list[tuple[str, str]]:
    if not bibtex_str or not bibtex_str.strip():
        return []
    entries: list[tuple[str, str]] = []
    block_pattern = re.compile(r"@\w+\s*\{([^,]+)", re.IGNORECASE)
    for key_match in block_pattern.finditer(bibtex_str):
        citation_id = key_match.group(1).strip()
        block_start = key_match.start()
        next_at = bibtex_str.find("@", key_match.end())
        block = bibtex_str[block_start : next_at if next_at != -1 else None]
        title_match = re.search(r"\btitle\s*=\s*\{", block, re.IGNORECASE)
        if not title_match:
            continue
        start = title_match.end()
        depth = 1
        i = start
        while i < len(block) and depth > 0:
            if block[i] == "{":
                depth += 1
            elif block[i] == "}":
                depth -= 1
            i += 1
        if depth == 0:
            title = block[start : i - 1].replace("\n", " ").strip()
            title = " ".join(title.split())
            if title:
                entries.append((citation_id, title))
    return entries


def _normalize_title_for_comparison(title: str) -> str:
    return " ".join(title.lower().strip().split())


def _get_duplicate_citations() -> list[tuple[str, str, str, str, str, str]]:
    """Same paper under different bibtex ids -> (task1, task2, id1, id2, raw_title_1, raw_title_2)."""
    by_title: dict[str, list[tuple[str, str, str]]] = defaultdict(list)
    for task_cls in mteb.get_tasks():
        if task_cls.metadata.bibtex_citation is None:
            continue
        for cid, title in _parse_bibtex_entries(task_cls.metadata.bibtex_citation):
            norm = _normalize_title_for_comparison(title)
            by_title[norm].append((f"task:{task_cls.metadata.name}", cid, title))

    for model_meta in mteb.get_model_metas():
        if model_meta.citation is None:
            continue
        for cid, title in _parse_bibtex_entries(model_meta.citation):
            norm = _normalize_title_for_comparison(title)
            by_title[norm].append((f"model:{model_meta.name}", cid, title))

    for benchmark in mteb.get_benchmarks():
        if benchmark.citation is None:
            continue
        for cid, title in _parse_bibtex_entries(benchmark.citation):
            norm = _normalize_title_for_comparison(title)
            by_title[norm].append((f"benchmark:{benchmark.name}", cid, title))

    duplicates = []
    for norm_title, items in by_title.items():
        id_to_raw = {cid: raw for _, cid, raw in items}
        id_to_item = {cid: item for item, cid, raw in items}
        if len(id_to_raw) < 2:
            continue
        unique_ids = sorted(id_to_raw)
        for id1, id2 in combinations(unique_ids, 2):
            duplicates.append(
                (
                    id_to_item[id1],
                    id_to_item[id2],
                    id1,
                    id2,
                    id_to_raw[id1],
                    id_to_raw[id2],
                )
            )

    all_titles = set(by_title.keys())
    for title, items in by_title.items():
        if items[0][2] in (
            "BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language",
            "BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language",
            "Introduction to the CoNLL-2002 Shared Task: Language-Independent Named Entity Recognition",
            "Introduction to the CoNLL-2003 Shared Task: Language-Independent Named Entity Recognition",
            "Overview of the TREC 2019 deep learning track",
            "Overview of the TREC 2020 deep learning track",
            "Overview of the TREC 2022 NeuCLIR track",
            "Overview of the TREC 2023 NeuCLIR Track",
            "MoD-Embedding: A Fine-tuned Multilingual Text Embedding Model",
            "Octen-Embedding-8B: A Fine-tuned Multilingual Text Embedding Model",
        ):
            continue
        use_titles = all_titles - {title}
        close_matches = difflib.get_close_matches(title, use_titles, n=2, cutoff=0.9)
        if close_matches:
            close_match_items = by_title[close_matches[0]][0]
            for item in items:
                duplicates.append(
                    (
                        item[0],
                        close_match_items[0],
                        item[1],
                        close_match_items[1],
                        item[2],
                        close_match_items[2],
                    )
                )
    return duplicates


def test_no_duplicate_citations_with_different_ids():
    """Ensure no task citations refer to the same paper under different BibTeX IDs."""
    duplicates = _get_duplicate_citations()
    assert not duplicates, (
        f"Found {len(duplicates)} duplicate citations (same paper, different BibTeX IDs). "
        "Unify citation keys or titles so each paper is cited once.\n\n"
        + "\n\n".join(
            f"--- Duplicate {i}: {item1} / {item2} ---\n"
            f"  id1 = {id1!r} ({item1})\n      title: {title1}\n"
            f"  id2 = {id2!r} ({item2})\n      title: {title2}"
            for i, (item1, item2, id1, id2, title1, title2) in enumerate(duplicates, 1)
        )
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class TweetTopicSingleClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TweetTopicSingleClassification",
        description="Topic classification dataset on Twitter with 6 labels. Each instance of TweetTopic comes with a timestamp which distributes from September 2019 to August 2021. Tweets were preprocessed before the annotation to normalize some artifacts, converting URLs into a special token {{URL}} and non-verified usernames into {{USERNAME}}. For verified usernames, we replace its display name (or account name) with symbols {@}.",
        dataset={
            "path": "mteb/TweetTopicSingleClassification",
            "revision": "b4280e921a2760ce34d2dd80a9e5dc8bcbf61785",
        },
        reference="https://arxiv.org/abs/2209.09824",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test_2021"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2019-09-01", "2021-08-31"),
        domains=["Social", "News", "Written"],
        task_subtypes=["Topic classification"],
        license="not specified",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{dimosthenis-etal-2022-twitter,
  address = {Gyeongju, Republic of Korea},
  author = {Antypas, Dimosthenis  and
Ushio, Asahi  and
Camacho-Collados, Jose  and
Neves, Leonardo  and
Silva, Vitor  and
Barbieri, Francesco},
  booktitle = {Proceedings of the 29th International Conference on Computational Linguistics},
  month = oct,
  publisher = {International Committee on Computational Linguistics},
  title = {{T}witter {T}opic {C}lassification},
  year = {2022},
}
""",
        superseded_by="TweetTopicSingleClassification.v2",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset["train"] = self.dataset["train_2021"]


class TweetTopicSingleClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TweetTopicSingleClassification.v2",
        description="Topic classification dataset on Twitter with 6 labels. Each instance of TweetTopic comes with a timestamp which distributes from September 2019 to August 2021. Tweets were preprocessed before the annotation to normalize some artifacts, converting URLs into a special token {{URL}} and non-verified usernames into {{USERNAME}}. For verified usernames, we replace its display name (or account name) with symbols {@}. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        dataset={
            "path": "mteb/tweet_topic_single",
            "revision": "a7904e26081f987da81ad2cc063e09e714e875d0",
        },
        reference="https://arxiv.org/abs/2209.09824",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test_2021"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2019-09-01", "2021-08-31"),
        domains=["Social", "News", "Written"],
        task_subtypes=["Topic classification"],
        license="not specified",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{dimosthenis-etal-2022-twitter,
  address = {Gyeongju, Republic of Korea},
  author = {Antypas, Dimosthenis  and
Ushio, Asahi  and
Camacho-Collados, Jose  and
Neves, Leonardo  and
Silva, Vitor  and
Barbieri, Francesco},
  booktitle = {Proceedings of the 29th International Conference on Computational Linguistics},
  month = oct,
  publisher = {International Committee on Computational Linguistics},
  title = {{T}witter {T}opic {C}lassification},
  year = {2022},
}
""",
        adapted_from=["TweetTopicSingleClassification"],
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from sentence_transformers import SentenceTransformer
from sentence_transformers.models import Pooling, Transformer

from mteb.models import ModelMeta
from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.types import PromptType

if TYPE_CHECKING:
    from collections.abc import Callable


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    return (
        f"{instruction}\nQuery: "
        if (prompt_type is None or prompt_type == PromptType.query) and instruction
        else "Represent this passage\npassage: "
    )


class BMRetrieverWrapper(InstructSentenceTransformerModel):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str | None = None,
        instruction_template: str
        | Callable[[str, PromptType | None], str]
        | None = None,
        max_seq_length: int | None = None,
        apply_instruction_to_passages: bool = True,
        padding_side: str | None = None,
        add_eos_token: bool = False,
        prompts_dict: dict[str, str] | None = None,
        **kwargs: Any,
    ):
        self.model_name = model_name
        self.instruction_template = instruction_template
        self.apply_instruction_to_passages = apply_instruction_to_passages
        self.prompts_dict = prompts_dict

        tokenizer_params = {}
        if add_eos_token:
            tokenizer_params["add_eos_token"] = add_eos_token
        if max_seq_length is not None:
            tokenizer_params["model_max_length"] = max_seq_length
        if padding_side is not None:
            tokenizer_params["padding_side"] = padding_side
        kwargs.setdefault("tokenizer_args", {}).update(tokenizer_params)
        kwargs.setdefault("config_args", {}).update(revision=revision)

        transformer = Transformer(
            model_name,
            device=device,
            **kwargs,
        )
        pooling = Pooling(
            transformer.get_word_embedding_dimension(), pooling_mode="lasttoken"
        )
        self.model = SentenceTransformer(modules=[transformer, pooling])


BMRETRIEVER_CITATION = """
@inproceedings{xu-etal-2024-bmretriever,
    title = "{BMR}etriever: Tuning Large Language Models as Better Biomedical Text Retrievers",
    author = "Xu, Ran and Shi, Wenqi and Yu, Yue and Zhuang, Yuchen and Zhu, Yanqiao and Wang, May Dongmei and Ho, Joyce C. and Zhang, Chao and Yang, Carl",
    booktitle = "Proceedings of the 2024 Conference on Empirical Methods in Natural Language Processing",
    month = "November",
    year = "2024",
    address = "Miami, Florida, USA",
    publisher = "Association for Computational Linguistics",
    pages = "22234--22254",
    url = "https://aclanthology.org/2024.emnlp-main.1241/"
}"""

# https://huggingface.co/datasets/BMRetriever/biomed_retrieval_dataset
BMRETRIEVER_TRAINING_DATA = {
    "FEVER",
    "MSMARCO",
    "NQ",
}

BMRetriever_410M = ModelMeta(
    loader=BMRetrieverWrapper,
    loader_kwargs=dict(
        model_args={"torch_dtype": torch.float32},
        instruction_template=instruction_template,
        padding_side="left",
        add_eos_token=True,
        apply_instruction_to_passages=True,
    ),
    name="BMRetriever/BMRetriever-410M",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="e3569bfbcfe3a1bc48c142e11a7b0f38e86065a3",
    release_date="2024-04-29",
    embed_dim=1024,
    n_parameters=353_822_720,
    n_embedding_parameters=51_511_296,
    memory_usage_mb=1349,
    max_tokens=2048,
    license="mit",
    reference="https://huggingface.co/BMRetriever/BMRetriever-410M",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=BMRETRIEVER_TRAINING_DATA,
    citation=BMRETRIEVER_CITATION,
)

BMRetriever_1B = ModelMeta(
    loader=BMRetrieverWrapper,
    loader_kwargs=dict(
        model_args={"torch_dtype": torch.float32},
        instruction_template=instruction_template,
        padding_side="left",
        add_eos_token=True,
        apply_instruction_to_passages=True,
    ),
    name="BMRetriever/BMRetriever-1B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="1b758c5f4d3af48ef6035cc4088bdbcd7df43ca6",
    release_date="2024-04-29",
    embed_dim=2048,
    n_parameters=908_759_040,
    n_embedding_parameters=103_022_592,
    memory_usage_mb=3466,
    max_tokens=2048,
    license="mit",
    reference="https://huggingface.co/BMRetriever/BMRetriever-1B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=BMRETRIEVER_TRAINING_DATA,
    citation=BMRETRIEVER_CITATION,
)

BMRetriever_2B = ModelMeta(
    loader=BMRetrieverWrapper,
    loader_kwargs=dict(
        model_args={"torch_dtype": torch.float32},
        instruction_template=instruction_template,
        padding_side="left",
        add_eos_token=True,
        apply_instruction_to_passages=True,
    ),
    name="BMRetriever/BMRetriever-2B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="718179afd57926369c347f46eee616db81084941",
    release_date="2024-04-29",
    embed_dim=2048,
    n_parameters=2_506_172_416,
    n_embedding_parameters=524_288_000,
    memory_usage_mb=9560,
    max_tokens=8192,
    license="mit",
    reference="https://huggingface.co/BMRetriever/BMRetriever-2B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=BMRETRIEVER_TRAINING_DATA,
    citation=BMRETRIEVER_CITATION,
)

BMRetriever_7B = ModelMeta(
    loader=BMRetrieverWrapper,
    loader_kwargs=dict(
        model_args={"torch_dtype": torch.float32},
        instruction_template=instruction_template,
        padding_side="left",
        add_eos_token=True,
        apply_instruction_to_passages=True,
    ),
    name="BMRetriever/BMRetriever-7B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="13e6adb9273c5f254e037987d6b44e9e4b005b9a",
    release_date="2024-04-29",
    embed_dim=4096,
    n_parameters=7_110_660_096,
    n_embedding_parameters=131_072_000,
    memory_usage_mb=27124,
    max_tokens=32768,
    license="mit",
    reference="https://huggingface.co/BMRetriever/BMRetriever-7B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=BMRETRIEVER_TRAINING_DATA,
    citation=BMRETRIEVER_CITATION,
)



from __future__ import annotations

from typing import TYPE_CHECKING

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

if TYPE_CHECKING:
    from collections.abc import Sequence

_CITATION = """
@misc{weller2025theoreticallimit,
  archiveprefix = {arXiv},
  author = {Orion Weller and Michael Boratko and Iftekhar Naim and Jinhyuk Lee},
  eprint = {2508.21038},
  primaryclass = {cs.IR},
  title = {On the Theoretical Limitations of Embedding-Based Retrieval},
  url = {https://arxiv.org/abs/2508.21038},
  year = {2025},
}"""


class LIMITRetrieval(AbsTaskRetrieval):
    k_values: Sequence[int] = (1, 2, 3, 5, 10, 20, 100, 1000)
    metadata = TaskMetadata(
        name="LIMITRetrieval",
        description="A simple retrieval task designed to test all combinations of top-2 documents. This version includes all 50k docs.",
        reference="https://github.com/google-deepmind/limit",
        dataset={
            "path": "orionweller/LIMIT",
            "revision": "48142cc741b04d0b4af370ade7e8b42430382670",
        },
        type="Retrieval",
        category="t2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="recall_at_2",
        modalities=["text"],
        date=("2025-08-28", "2025-08-28"),
        domains=["Fiction"],
        task_subtypes=["Question answering"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=_CITATION,
    )


class LIMITSmallRetrieval(AbsTaskRetrieval):
    k_values: Sequence[int] = (1, 2, 3, 5, 10, 20, 100, 1000)
    metadata = TaskMetadata(
        name="LIMITSmallRetrieval",
        description="A simple retrieval task designed to test all combinations of top-2 documents. This version only includes the 46 documents that are relevant to the 1000 queries.",
        reference="https://github.com/google-deepmind/limit",
        dataset={
            "path": "orionweller/LIMIT-small",
            "revision": "ff4a8f2476ae77476c1912f1f3cb5bb5f2d766d4",
        },
        type="Retrieval",
        category="t2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="recall_at_2",
        modalities=["text"],
        date=("2025-08-28", "2025-08-28"),
        domains=["Fiction"],
        task_subtypes=["Question answering"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=_CITATION,
    )
